\ FormObject.f

\ Needs Resources.f 	\ to Add Resources	\ DB 2009-0609

: RefreshForms  ( -- )  \ refresh all forms for a change in color settings
\ forms will be automatically updated but still available from any form popup menu
                #Forms 0= ?exit         \ no forms open
                COLOR_BTNFACE Call GetSysColor NewColor: FormColor      \ get new system color
                Link#: FormList >r                                      \ save current link
                >FirstLink: FormList                                    \ start from the beginning
                #Forms 0                                                \ for each open form
                ?do     Data@: FormList dup to ThisForm
                        if      Paint: ThisForm                         \ repaint
                        then    >NextLink: FormList                     \ next form
                loop    r> >Link#: FormList ;

\ Popup menus for forms
PopUpBar FormPopup
PopUp ""
         MenuItem "Properties"            doform ;
         MenuSeparator
         MenuItem "Test"                  dotest ;
         MenuSeparator
         MenuItem "Group Action"          doGroupAction ;
         MenuSeparator
         MenuItem "View/Edit Source"      doEditor ;
         MenuItem "Save (.ff)"        	  doSave ;
         MenuItem "Compile (.frm)"    	  doWrite ;
         :MenuItem mnu_lock "View Only"   Lock: ActiveForm ;
         MenuSeparator
         MenuItem "Refresh"               RefreshForms ;
         MenuSeparator
         MenuItem "Delete Controls"       doDeleteControls ;
         MenuSeparator
         MenuItem "Close"                 ActiveForm doCloseForm ;
Endbar

PopupBar ChildPopup

Popup ""
         MenuItem "Properties"        doedit ;
         SubMenu "Align"
                MenuItem "to grid point"  AlignGridXY ;
                MenuItem "to grid's vertical plane" AlignGridY ;
                MenuItem "to grid's horizontal plane" AlignGridX ;
         EndSubMenu
         MenuItem "Size sequential Controls" AutoSize  ;
         MenuItem "Arrange sequential Controls vertically" ArrangeVertical ;
         MenuItem "Arrange sequential Controls horizontally" ArrangeHorizontal ;
         MenuSeparator
         MenuItem "Copy Control"      CopyControl ;	\ added to Copy-Paste Controls DB 2009-04
         MenuItem "Delete Control"    doDelete ;
         MenuItem "Change Type"       ChangeControl ;
         MenuItem "Change to DrawBox" ChangeToDrawBox ;   \ added DB 2010-0521
         MenuItem "Change to ScrollTextBox" ChangeToScrollTextBox ;   \ added DB 2012-0422
         MenuSeparator
         :MenuItem mnu_font "Change Font"  ChangeControlFont ;
         :MenuItem mnu_dfont "Reset to default Font" ResetControlFont ;
         MenuSeparator
         MenuItem "Bring to Front"    doMoveToFront ;
         MenuItem "Move to Back"      doMoveToBack  ;
EndBar

: 2tabs              ( -- )
                     2 +tabs ;

1 value #timers		\ for DeleteTurnkeyLoader	\ DB 2009-0507

\ Temporary Buffer for Bitmaps				\ DB 2009 0729

1024 constant BitmapsBufferlen
0 Variable BitmapsUsed BitmapsBufferlen allot
value Bitmapbufferpointer
: Erase-Bitmapbuffer BitmapsUsed BitmapsBufferlen BL fill 0 to Bitmapbufferpointer ;
: >Bitmapbuffer ( a n -- )
   >r >r BitmapsUsed Bitmapbufferpointer + 1+ 
   r> swap r> dup Bitmapbufferpointer + 1+ dup to Bitmapbufferpointer 
   BitmapsBufferlen <= if cmove else 3drop then
;

: Skip-BL	  ( a n -- a n )	\ skips leading BL's in String	\ DB 2009
		dup 0> if
		2dup bounds do i c@ BL <= if 1- swap 1+ swap else leave then loop 
		       then
;

: GetWord	  ( a n -- a n )	\ gets first word in string	\ DB 2009
		dup 0> if
		Skip-BL over -rot 0 -rot bounds 
                         do i c@ BL <= if leave else 1+ then loop 
		       then
;

102 value BMP-ID

: AddBitmap ( ID a n -- ) 
\ 3dup  cr 2DUP type . . . 	\ for testing only DB 2009 0729
  dup 5 < if   3drop
          else AddBitmap 
          then
;

: (AddBitmap)		( a n -- a n flg )	\ DB 2009 0729
	dup if   dup >r GetWord dup
			if   2dup + >r
		 	     BMP-ID dup 1+ to BMP-ID -rot "path-file drop AddBitmap 
		 	     r> r> true
			else r> drop false
			then
	    else false
	    then
;

: AddBitmaps 		( -- ) 	 \ add bitmaps for EXE	\ DB 2009 0729
    \            #controls 0= ?exit
                BitmapsUsed Bitmapbufferpointer 
	\	128 1 do (AddBitmap) 0= if leave then loop 2drop
        \ changed to 255 \ DB 2013-0722
		255 1 do (AddBitmap) 0= if leave then loop 2drop
;

\ Define Form Object class for ForthForm


:Class Form  <Super Window

Record: Info
        int frmWidth
        int frmHeight
        int frmXPos
        int frmYPos
  maxstring FormFunctionMax 2 * - 100 - bytes frmTitle     \ changed DB 2012-0414 + 2012-0428
FormFunctionMax bytes frmPaintFunction   \ DB 2012-0428
FormFunctionMax bytes frmSizeFunction    \ DB 2012-0428 
  100 bytes frmMenuBar             \ added DB 2012-0414
   32 bytes frmName
        int controlcount
       byte FormFlags
     1 bits Modal
     1 bits SaveScreen?
     2 bits FormSuperClass  \ 0 = dialogwindow, 1 = childwindow, 2 = mdi dialog window, 3 = Super Window \ DB 2012-0407
     1 bits Childstate
     1 bits with-statusbar
     1 bits with-menubar                  \ DB 2012-0414
     1 bits with-interpreter              \ DB 2012-0511
 \    1 bits ReservedFlags                \ used up now ! DB 2012-0511
FormFunctionMax bytes frmInitFunction     \ DB 2009-0502 - changing the number of bytes
FormFunctionMax bytes frmCloseFunction    \ DB 2009-0502 - results in incompatability of
FormFunctionMax bytes frmDoneFunction     \ DB 2009-0502 - former Forms / .ff-Files !
FormNeedsMax    bytes frmNeedsFunction    \ DB 2009-0527 - added for needs in top of form
    11 bytes Reserved    	          \ for future features
;RecordSize: sizeof(Info)

Rect FormBox    \ for drawing boxes
max-path bytes formname   \ filename for this form
int ActiveControl         \ selected control
int ControlList           \ pointer to list of controls for this form
int modified?             \ was this form modified ?
int ThisControl           \ used temporarily for newly created control
int ThatControl           \ used temporarily tocopy control	\ DB 2009-0428
int sizing?
int moving?
int cclicked?           \ control clicked ?
int selecting?          \ true if selecting a group of controls
LTGRAY value BackGroundColor  \ duh!
Point upperleft         \ bounding coordinates
Point lowerright        \ for active control
File FormFile           \ file name for this window
5 constant hsize        \ square handle size
int lastx
int lasty
int firsttime?          \ are we sizing for the first time?
int locked?
MultistatusBar frmStatusbar

int timer   		\ for DeleteTurnkeyLoader	\ DB 2009-0507


256 constant array-size  \ holds count of each control type created for naming - changed from 32 to 256 DB 2009-0314
\ purposes. Limit of 255 of any one type per form. If more is needed increase the
\ constant value \ does not work because arraycounter is only a Byte ! DB 2009-0430
array-size bytes array()

0 value BitmapUsed	\ true if Bitmap is used	\ added DB 2009-0501
0 value GraphicsUsed	\ true if Graphics are used	\ added DB 2010-0522

: inc-array     ( index -- )
                1- array() + cincr ;

: dec-array     ( index -- )
                1- array() + cdecr ;

: get-array     ( index -- value )
                1- array() + c@ ;

:M Array():     ( n -- val )
                get-array ;M

: ShowModified     ( -- )
                modified?
                if      z" Modified"
                else    z" "
                then    3 SetText: statuswindow ;

: modified      ( -- )
                true to modified? ShowModified
                UpdateSystem ;

:M SendMessage: ( l w m -- res )
                hwnd Call SendMessage ;M

: ShowFormName  ( -- )
                frmName count asciiz 4 SetText: statuswindow ;

:M UpdateStatus:  ( -- ) \ refresh display of control information
                ActiveControl 0=
                if  ClearStatusWindow ShowModified
                    ShowFormName
                    exitm
                then    Origin: ActiveControl ShowPosition
                Dimensions: ActiveControl ShowSize
                GetName: ActiveControl asciiz 0 SetText: statuswindow
                ShowModified
                ShowFormName
                ;M

: IsActiveControl ( ctrl -- )
		dup to ActiveCtrl	\ DB 2009-0504 for Editfunction-Title
                to ActiveControl
                self to Activeform
                UpdatePropertyWindow
                UpdateStatus: self ;

: NoActiveControl  ( -- )
                0 to ActiveControl
                ClearStatusWindow
                ShowModified    \ nevertheless show form status
                ShowFormName ;

: #controls      ( -- n )
                ControlList
                if      Data@: ControlList
                        if      #Links: ControlList
                        else    0
                        then
                else    0
                then    ;

: ?controlnumber { <object> -- n } \ given object address return position in list
                <object> 0= ControlList 0= or
                if      false exit
                then    >FirstLink: ControlList
                #controls 1+ 1
                ?do     Data@: ControlList <object> =
                        if     i unloop
                               exit
                        then   >NextLink: ControlList
                loop    0 ;

:M ?ControlNumber:      ( ctrl -- n )
                ?controlnumber ;M

: DisposeControls   ( -- )
                #controls 0= ?exit
                ControlList
                if      >FirstLink: ControlList
                        begin       Data@: ControlList dup to ThisControl
                        while       Free: ThisControl
				    \ ThisControl Dispose
                                    0 Data!: ControlList
                                    DeleteLink: ControlList
                        repeat      ControlList Dispose
                        0 to ControlList
                then    0 to ActiveControl ;

: NewControl (  -- addr )
   New> ControlObject dup to ThisControl ;

: (AddNewControl) (  --  )
                ControlList 0=
                if      New> Linked-List to ControlList
                then    Data@: ControlList
                if      AddLink: ControlList
                then    NewControl Data!: ControlList ;

: AddNewControl ( -- )
                ['] (AddNewControl) catch ExitOnError ;


: fonttype?     ( ctrltype -- f )
                dup 0= ?exit
                |   TypePushbutton
                    TypeGroupBox
                    TypeCheckBox
                    TypeRadioButton
                    TypeLabel
                    TypeTextBox
                    TypeMultiLineBox
                    TypeScrollTextBox
                    TypeListBox
                    TypeComboBox
                    TypeComboListBox
                    TypeMultiListBox
                    TypeGeneric     \ in case we just want a font
                |if    true
                else   false
                then   ;

: ?mnu_font     ( -- )  \ according to type of control allow changing font
                ActiveControl dup
                if     GetType: [ ] fonttype?
                then   dup Enable: mnu_font Enable: mnu_dfont ;

:M AlignGridXY: ( -- )  \ align control to grid's x,y plane
                ActiveControl 0= ?exitm
                Origin: ActiveControl >r gridspacing / gridspacing *
                r> gridspacing / gridspacing *
                SetOrigin: ActiveControl
                Update: ActiveControl modified ;M

:M AlignGridY:  ( -- ) \ align to grid's vertical plane
                ActiveControl 0= ?exitm
                Origin: ActiveControl >r gridspacing / gridspacing *
                r> SetOrigin: ActiveControl
                Update: ActiveControl modified ;M

:M AlignGridX:  ( -- )  \ align to grid's horizontal plane
                ActiveControl 0= ?exitm
                Origin: ActiveControl gridspacing / gridspacing *
                SetOrigin: ActiveControl
                Update: ActiveControl modified ;M

: islastlink?   ( -- f )
                Link#: ControlList >r
                ActiveControl ?controlnumber >Link#: ControlList
                LastLink?: ControlList
                r> >Link#: ControlList ;

: activecontrol#        ( -- n )
                ActiveControl ?controlnumber ;

\ size a group of controls to same size of active control
\ terminate when group ends
:M AutoSize:    { \ awidth aheight atype ctrl -- }
                ActiveControl 0= #controls 2 < or islastlink? or ?exitm
                Dimensions: ActiveControl to aheight to awidth
                GetType: ActiveControl to atype
                #controls activecontrol# - 0     \ to end of list
                do      >NextLink: ControlList
                        Data@: Controllist to ctrl
                        GetType: ctrl atype =
                        if      awidth aheight SetDimensions: ctrl
                                Update: ctrl
                        else    leave            \ similarity ends
                        then
                loop    modified ;M

\ these two definitions arrange all similiar controls evenly spaced
\ assumes window is big enough



:M ArrangeVertical:     { \ height ctrl x y atype -- }
\ arrange orderly all sequential controls of this type
		GetSpacing
                ActiveControl 0= #controls 2 < or islastlink? or ?exitm
                GetType: ActiveControl to atype
                Dimensions: ActiveControl nip to height
                Origin: ActiveControl height + spacing + to y to x
                #controls activecontrol# - 0
                ?do     >NextLink: ControlList
                        Data@: ControlList to ctrl
                        GetType: ctrl atype =
                        if      x y SetOrigin: ctrl
                                Update: ctrl
                                y height + spacing + to y
                        else    leave    \ end at first change
                        then
                loop    modified ;M

:M ArrangeHorizontal: { \ width ctrl x y atype -- }
\ arrange orderly all sequential controls of this type
		GetSpacing
                ActiveControl 0= #controls 2 < or islastlink? or ?exitm
                GetType: ActiveControl to atype
                Dimensions: ActiveControl drop to width
                Origin: ActiveControl to y width + spacing + to x
                #controls activecontrol# - 0
                ?do     >NextLink: ControlList
                        Data@: ControlList to ctrl
                        GetType: ctrl atype =
                        if      x y SetOrigin: ctrl
                                Update: ctrl
                                x width + spacing + to x
                        else    leave    \ end at first change
                        then
                loop    modified ;M

: inbox?        { ctrl -- f } \ f = true if top left corner of ctrl is in box
                Origin: ctrl >r Left: FormBox Right: FormBox between
                r> Top: FormBox Bottom: FormBox between and ;

:M GroupAlignLeft:  { \ ctrl -- }   \ align grouped controls to left edge of box
                selecting? not ?exitm
                #controls 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Left: FormBox Origin: ctrl nip SetOrigin: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M

:M GroupAlignRight: { \ ctrl -- }   \ align grouped controls to right edge of box
                selecting? not ?exitm
                #controls 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Origin: ctrl drop Dimensions: ctrl drop +   ( x + w )
                                Right: FormBox swap - Origin: ctrl >r + r>
                                SetOrigin: ctrl Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M

: (GroupAlignTop:)   { \ ctrl -- }   \ align grouped controls to top edge of box
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Origin: ctrl drop Top: FormBox SetOrigin: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;

:M GroupAlignTop:   ( -- )  \ align grouped controls to top edge of box
                selecting? not ?exitm
                #controls 0= ?exitm
                (GroupAlignTop:) ;M

:M GroupAlignBottom:    { \ ctrl -- }   \ align grouped controls to bottom edge of box
                selecting? not ?exitm
                #controls 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Origin: ctrl nip Dimensions: ctrl nip +  ( y + h )
                                Bottom: FormBox swap - >r Origin: ctrl r> +
                                SetOrigin: ctrl Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M

: sizecontrol   { n \ ctrl this w h flag -- } \ n=1=tallest,2=shortest,3=widest,4=narrowest
\ this test code can be simplified, get rid of all these "if "then"!
                >FirstLink: ControlList
                0 to this   \ will hold desired control
                n 2 > to flag
                #controls 0
                do  Data@: ControlList dup to ctrl inbox?
                    if  Dimensions: ctrl to h to w
                        this 0= \ not yet have a value
                        if  flag
                            if  w   \ we want width
                            else    h
                            then    to this
                        else    flag
                            if  w
                            else    h
                            then    dup this n 2 mod
                            if  >
                            else    <
                            then
                            if  to this
                            else    drop
                            then
                        then
                    then    >NextLink: ControlList
                loop    \ we should now have the one we want
                this 0= ?exit   \ leave if we don't
                >FirstLink: ControlList
                #controls 0
                do  Data@: ControlList dup to ctrl inbox?
                    if  Dimensions: ctrl flag
                        if  nip this swap
                        else    drop this
                        then    SetDimensions: ctrl
                        Update: ctrl modified
                    then    >NextLink: ControlList
                loop    UpdateStatus: self UpdatePropertyWindow ;

:M GroupSizeTallest:    ( -- )
            selecting? not ?exitm
            #controls 0= ?exitm
            Link#: ControlList >r
            1 sizecontrol
            r> >Link#: ControlList Paint: self ;M

:M GroupSizeShortest:   ( -- )
            selecting? not ?exitm
            #controls 0= ?exitm
            Link#: ControlList >r
            2 sizecontrol
            r> >Link#: ControlList Paint: self ;M

:M GroupSizeWidest: ( -- )
            selecting? not ?exitm
            #controls 0= ?exitm
            Link#: ControlList >r
            3 sizecontrol
            r> >Link#: ControlList Paint: self ;M

:M GroupSizeNarrowest:  ( -- )
            selecting? not ?exitm
            #controls 0= ?exitm
            Link#: ControlList >r
            4 sizecontrol
            r> >Link#: ControlList Paint: self ;M

create spad 2000 allot    \ temporary sort pad, 24 controls maximum at a time
			  \ changed from 100 allot to 2000 allot to get hopefully 480 controls

: findcontrols      ( -- ) \ find all controls in box, save to spad
            spad off
            >FirstLink: ControlList
            #controls 0
            do  Data@: ControlList dup inbox?
                if  spad lcount cells+ !
                    1 spad +!
                else    drop
                then
                >NextLink: ControlList
            loop  ;

: sortcontrols  { n -- }    \ n=0 vertical sort else horizontal sort ascending
\ Simple sort algorithm, e.g 4 3 2 1 - test 4,3 then 3,2 then 2,1 - swap if
\ necessary after compare, do this for (n items - 1) times
            spad @ 2 < ?exit
            spad @ 1- 0          \ n-1 iterations
            do  spad lcount 1- cells bounds
                do  i @ Origin: [ ] n
                    if      drop    ( x )
                    else    nip     ( y )
                    then    i cell+ @ Origin: [ ] n
                    if  drop
                    else    nip
                    then    >
                    if  i @ i cell+ tuck @ i ! swap !   \ swap memory
                    then
               cell +loop
            loop ;

: (GroupArrangeVertical:) { \ ctrl x y h -- }    \ starting at first control
	GetSpacing
            Link#: ControlList >r
            findcontrols
            0 sortcontrols
            r> >Link#: ControlList
            spad @ 2 < 0=
            if	spad lcount >r @ to ctrl
            	Dimensions: ctrl nip to h
            	Origin: ctrl h + spacing + to y to x	
            	spad 2 cells+ r> 1- cells bounds
            	do  x y i @ >r SetOrigin: [ r@ ]
                	Update: [ r@ ]
                	Dimensions: [ r> ] nip y + spacing + to y
          		cell  +loop   modified UpdateStatus: self UpdatePropertyWindow
            then Paint: self ;

: (GroupArrangeHorizontal:) { ctrl x y w -- }    \ starting at first control
	GetSpacing
            Link#: ControlList >r
            findcontrols
            1 sortcontrols
            r> >Link#: ControlList
            spad @ 2 < 0=
            if	spad lcount >r @ to ctrl
            	Dimensions: ctrl drop to w
            	Origin: ctrl to y w + spacing + to x
            	spad 2 cells+ r> 1- cells bounds
            	do  x y i @ >r SetOrigin: [ r@ ]
                	Update: [ r@ ]
                	Dimensions: [ r> ] drop x + spacing + to x
           		cell +loop   modified UpdateStatus: self UpdatePropertyWindow
            then Paint: self ;

0 value RowHeight
0 value MoveRow

: MoveObjects	  ( -- n ) GetColoums MoveRow 1-  0 max * ;


:M GroupMoveDown:   { \ ctrl val -- }   \ shift down by specified value
                selecting? not ?exitm
                #controls 0= ?exitm
                GetValue: frmAction to val
                val 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Origin: ctrl val + SetOrigin: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M


: (GroupSizeToBox:)  { \ bx by bw bh ctrl -- }
            Link#: ControlList >r
            >FirstLink: ControlList
            #controls 0
            do  Data@: ControlList dup to ctrl inbox?
                if  Left: FormBox Origin: ctrl nip SetOrigin: ctrl
                    Right: FormBox Left: FormBox - ( new width) 
		    ColoumsForArray dup 1 >		\ added for button-arrays DB 2009-0305
			if   dup >r 1- spacing * - r> /	\ added for button-arrays DB 2009-0305
			else drop
			then Dimensions: ctrl nip SetDimensions: ctrl
                    Update: ctrl modified
                then    >NextLink: ControlList
            loop    r> >Link#: ControlList
            UpdateStatus: self UpdatePropertyWindow
            Paint: self ;

: (GroupHeightToBoxes:)  { \ ctrl val -- } \ added for button-arrays DB 2009-0306
	Bottom: FormBox Top: FormBox - abs   \ height
	RowsForArray dup >r 1- spacing * - r> / to val
                val 
	   if	val to RowHeight
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Function-Text count IsFunction: ctrl		\ added for cloning DB 2009-0318
				EdFunction-Text count IsEdFunction: ctrl	\ dito
				Requirements-Text count IsRequirements: ctrl	\ added for cloning DB 2009 0611
				Bitmap-Text count IsBitmap: ctrl		\ for Arrays Bitmap Cloning DB 2009-0501
				Dimensions: ctrl drop val SetDimensions: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow 
	   then ;

: (GroupFunctionToBoxes:)  { \ ctrl -- } \ added for horizontal / vertical cloning DB 2009-0424
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Function-Text count IsFunction: ctrl
				EdFunction-Text count IsEdFunction: ctrl
				Requirements-Text count IsRequirements: ctrl	\ added for cloning DB 2009 0611
				Bitmap-Text count IsBitmap: ctrl		\ for Arrays Bitmap Cloning DB 2009-0501
				Dimensions: ctrl ControlWidth ?dup if rot drop swap then ControlHeigth ?dup if nip then SetDimensions: ctrl	\ for Array-Corrections DB 2009-0501
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;

0 value BoxWidth
0 value Boxheight
0 value BoxX
0 value BoxY
0 value verticalBoxes

: (GroupArrangeBoxes:) { ctrl x y w n -- }    \ starting at first control		\ added DB 2009-0423
	RowColoumsArray?
        selecting? not 
        #controls 2 < or 0= and
	 if GetSpacing
            Link#: ControlList >r
            findcontrols
            1 sortcontrols
            r> >Link#: ControlList
            spad @ 2 < 0=
            if	spad lcount >r @ to ctrl 
            	Dimensions: ctrl to Boxheight to w ( BoxWidth )
	    	1 to n
	    	verticalBoxes
            	   if	Origin: ctrl dup to BoxY Boxheight + spacing + to y dup to BoxX to x
            		spad 2 cells+ r> 1- cells bounds
            		do  n 1+ to n x y i @ >r SetOrigin: [ r@ ]
                	    Update: [ r@ ]
                	    Dimensions: [ r> ] drop
			    n RowsForArray /mod drop
				if   drop y Boxheight + spacing +
				else drop x w + spacing + to x BoxY
				then to y
               		    cell +loop  
             	   else	Origin: ctrl dup to BoxY to y dup to BoxX w + spacing + to x
            		spad 2 cells+ r> 1- cells bounds
            		do  n 1+ dup to n x y i @ >r SetOrigin: [ r@ ]
                	    Update: [ r@ ]
                	    Dimensions: [ r> ] drop
		 	    n ColoumsForArray /mod drop
				if   x + spacing + 
				else drop y Boxheight + spacing + to y BoxX
				then to x
                	    cell +loop  
		   then	modified UpdateStatus: self UpdatePropertyWindow
            then Paint: self 
	 then ;

:NoName  ( -- )				\ DB 2009-0430
    1 SetValue: txtRows 1 SetValue: txtColoums	\ for security. Better: disable!
    SwitchToSelect false to lastType ; is DisableArray

:M GroupSizeToBox:  ( -- )			\ Making an Array, new arranged DB 2009-0423
	false to verticalBoxes 
	GetSpacing 2 to MoveRow 
	RowColoumsArray? dup
	   if 	#controls 1+ < then
           if 	selecting? not ?exitm
                #controls 2 < ?exitm		\ changed from 0= to 2 < DB 2009-0424
		(GroupHeightToBoxes:)  
		(GroupAlignTop:)  
		(GroupSizeToBox:)  
		(GroupArrangeBoxes:)
		DisableArray
           else	selecting? not ?exitm
            	#controls 0= ?exitm
		(GroupSizeToBox:)
	   then ;M


:M GroupArrangeVertical: ( -- )   \ starting at first control
	RowColoumsArray? dup
	if #controls 1+ < then
		   if   true to verticalBoxes 
			selecting? not ?exitm
                	#controls 2 < ?exitm		\ changed from 0= to 2 < DB 2009-0424
			(GroupFunctionToBoxes:)
			(GroupArrangeBoxes:)
			DisableArray
		   else	selecting? not ?exitm
            		#controls 2 < ?exitm
			(GroupArrangeVertical:)
		   then ;M

:M GroupArrangeHorizontal: ( -- )    \ starting at first control
	RowColoumsArray? dup
	if #controls 1+ < then
		   if	false to verticalBoxes 
			selecting? not ?exitm
                	#controls 2 < ?exitm		\ changed from 0= to 2 < DB 2009-0424
			(GroupFunctionToBoxes:)
			(GroupArrangeBoxes:)
			DisableArray
		   else selecting? not ?exitm
            		#controls 2 < ?exitm
			(GroupArrangeHorizontal:)
		   then ;M


:M GroupMoveUp:   { \ ctrl val -- }   \ shift up by specified value
                selecting? not ?exitm
                #controls 0= ?exitm
                GetValue: frmAction to val
                val 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Origin: ctrl val - SetOrigin: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M

:M GroupMoveLeft:   { \ ctrl val -- }   \ shift left by specified value
                selecting? not ?exitm
                #controls 0= ?exitm
                GetValue: frmAction to val
                val 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Origin: ctrl val negate under+ SetOrigin: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M

:M GroupMoveRight:   { \ ctrl val -- }   \ shift right by specified value
                selecting? not ?exitm
                #controls 0= ?exitm
                GetValue: frmAction to val
                val 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Origin: ctrl val under+ SetOrigin: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M

:M GroupHeightUp:   { \ ctrl val -- }   \ increase height by specified value
                selecting? not ?exitm
                #controls 0= ?exitm
                GetValue: frmAction to val
                val 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Dimensions: ctrl val + SetDimensions: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M

:M GroupHeightDown:   { \ ctrl val -- }   \ reduce height by specified value
                selecting? not ?exitm
                #controls 0= ?exitm
                GetValue: frmAction to val
                val 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Dimensions: ctrl val - SetDimensions: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M

:M GroupWidthUp:   { \ ctrl val -- }   \ increase width by specified value
                selecting? not ?exitm
                #controls 0= ?exitm
                GetValue: frmAction to val
                val 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Dimensions: ctrl val under+ SetDimensions: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M

:M GroupWidthDown:   { \ ctrl val -- }   \ reduce width by specified value
                selecting? not ?exitm
                #controls 0= ?exitm
                GetValue: frmAction to val
                val 0= ?exitm
                Link#: ControlList >r
                >FirstLink: ControlList
                #controls 0
                do      Data@: ControlList dup to ctrl inbox?
                        if      Dimensions: ctrl val negate under+ SetDimensions: ctrl
                                Update: ctrl modified
                        then    >NextLink: ControlList
                loop    r> >Link#: ControlList Paint: self UpdateStatus: self
                UpdatePropertyWindow ;M

:M GetData:     ( -- addr cnt )
                Info sizeof(Info)
                ;M

:M SetData:     ( addr cnt -- )
                sizeof(Info) min 0max Info swap move
                ;M

:M GetCount:    ( -- cnt )
                controlcount
                ;M

:M SetCount:    ( count -- )
                to controlcount
                ;M

: +hsize        { x y  -- x2 y2 }
                hsize x + hsize y + ;

: controlbox    ( -- )  \ set the boundaries for the active control
                GetWindowRect: ActiveControl
                SetPoint: lowerright
                SetPoint: upperleft ;

: topleft       ( -- tlx tly brx bry )  \ return rect of topleft handle
                GetPoint: upperleft 7 -   \ box points must be set
                2dup +hsize ;

: topright      ( -- tlx tly brx bry )
                lowerright.x 4 - upperleft.y 7 -
                2dup +hsize ;

: bottomleft    ( -- tlx tly brx bry )
                upperleft.x lowerright.y 2 +
                2dup +hsize ;

: bottomright   ( -- tlx tly brx bry )
                lowerright.x 4 - lowerright.y 2 +
                2dup +hsize ;

\ if mouse is in handle set handle id
\ 1=topleft, 2=topright, 3=bottomleft, 4=bottomright
0 value hid

: inhandle?     ( -- f )        \ is mouse in a sizing handle?
                0 to hid        \ reset
                ActiveControl 0=
                if      false exit
                then    controlbox
                topleft     >r swap >r mousex -rot between mousey r> r> between and ?dup
                if      1 to hid exit   then
                topright    >r swap >r mousex -rot between mousey r> r> between and ?dup
                if      2 to hid exit   then
                bottomleft  >r swap >r mousex -rot between mousey r> r> between and ?dup
                if      3 to hid exit   then
                bottomright >r swap >r mousex -rot between mousey r> r> between and ?dup
                if      4 to hid exit   then
                false ;

:M ShowSizingHandles:   { flag \ fillcolor -- }
\ if flag = true show handles, else erase them
               ActiveControl 0<> moving? not and
               if       flag
                        if      LTBLUE
                        else    BackGroundColor
                        then    to fillcolor
                        get-dc
                        controlbox
                        topleft     fillcolor FillArea: dc \ 5x5 handles
                        topright    fillcolor FillArea: dc \ at each
                        bottomleft  fillcolor FillArea: dc \ of four
                        bottomright fillcolor FillArea: dc \ corners
                        release-dc
                then    ;M

: ShowHandles   ( -- )
                true ShowSizingHandles: self ;

: HideHandles   ( -- )
                false ShowSizingHandles: self ;

: (drawBox)     { n  -- }
                get-dc
                addr: dc
                SetDC: FormBox
                n
                if     DrawDotted: FormBox
                else   DrawNormal: FormBox
                then   release-dc ;

: drawbox       ( -- )
                0 (drawbox) ;


: eraseBox      ( -- ) \ really same as drawbox
                drawbox ;


: drawdottedbox    ( -- )
                1 (drawbox) ;

: ShowBox       ( left top right bottom -- )
                mousex to lastx mousey to lasty  \ save mouse first
                eraseBox
                4dup SetRect: FormBox drawBox
                selecting? not
                if      2over ShowPosition
                        rot - -rot swap - swap ShowSize
                else    4drop
                then    ;

\ The following four routines adjust the selection rectangle using the arrow keys
\ Each press of the arrow key adjusts by 4, but can be modified
\ Handy when a group of controls has been moved out of the selection rectangle
: adjust-left   ( -- )
                selecting? not ?exit
                Left: FormBox cell- Top: FormBox Right: FormBox Bottom: FormBox
                SetRect: FormBox Paint: self ;

: adjust-right  ( -- )
                selecting? not ?exit
                Left: FormBox Top: FormBox Right: FormBox cell+ Bottom: FormBox
                SetRect: FormBox Paint: self ;

: adjust-up     ( -- )
                selecting? not ?exit
                Left: FormBox Top: FormBox cell- Right: FormBox Bottom: FormBox
                SetRect: FormBox Paint: self ;

: adjust-down   ( -- )
                selecting? not ?exit
                Left: FormBox Top: FormBox Right: FormBox Bottom: FormBox cell+
                SetRect: FormBox Paint: self ;

: sizebottomright     { \ l t r b -- }
                ActiveControl 0= ?exit
                Left: FormBox to l
                Top: FormBox to t
                mousex lastx - Right: FormBox +  to r
                mousey lasty - Bottom: FormBox +  to b
                l t r b ShowBox ;

: sizetopleft      ( -- )
                ActiveControl 0= ?exit
                mousex lastx - Left: FormBox +
                mousey lasty - Top: FormBox +
                Right: FormBox Bottom: FormBox
                ShowBox ;

: sizetopright  ( -- )
                ActiveControl 0= ?exit
                Left: FormBox
                mousey lasty - Top: FormBox +
                mousex lastx - Right: FormBox +
                Bottom: FormBox
                ShowBox ;

: sizebottomleft  ( -- )
                ActiveControl 0= ?exit
                mousex lastx - Left: FormBox +
                Top: FormBox
                Right: FormBox
                mousey lasty - Bottom: FormBox +
                ShowBox ;

: incontrol?    { ctrl \ left top right bottom -- }   \ is mouse in control object ?
                GetWindowRect: ctrl to bottom to right to top to left
                mousex left right between
                mousey top bottom between and ;

: ControlClicked?       ( -- ctrl | 0 )
                false        \ default
                #controls 0= ?exit
                >LastLink: ControlList          \ start from back
                #controls 0
                do      Data@: ControlList dup incontrol?
                        if      nip unloop exit
                        else    drop
                        then    >PrevLink: ControlList
                loop    ;

: clicked       ( -- )
                hwnd call SetCapture drop
		one-click?
		if	ButtonID IDC_BITMAP IDC_Generic between
                	if      false ButtonID   CheckButton: TheControlToolBar     \ uncheck it
                        	true  IDC_SELECT CheckButton: TheControlToolBar     \ set default
                	then    0 to ButtonID  \ clear buttonid
		then	HideHandles
                locked?
                if      ControlClicked? IsActiveControl         \ set temporary for status display
                        0 to ActiveControl                      \ clear when done
                        exit
                then    mousex to lastx mousey to lasty
                newcontrol?
                if      mousex mousey 2dup        \ starting size
                        SetRect: FormBox
                        drawBox
                        NoActiveControl exit
                then    sizing? ?exit
                inhandle?
                if      GetWindowRect: ActiveControl
                        SetRect: FormBox
                        drawBox
                        true to sizing? exit
                then    ControlClicked? ?dup
                if      IsActiveControl
                        GetWindowRect: ActiveControl SetRect: FormBox
                        drawbox
                        true to cclicked? false to selecting?
                        hand-cursor
                        ShowHandles exit
                then    selecting?
                if      erasebox            \ remove old box
                then    true to selecting?          \ selecting controls
                mousex mousey 2dup SetRect: FormBox     \ no initial size
                drawbox ;                   \ keep control active

: CheckBitmap	( ControlType -- )			\ added DB 2009-0501
		dup  TypeBitmapButton =
                over TypeIconButton   = or
                swap TypeStaticBitmap = or
		if   true to BitmapUsed
		then ;

: CheckDrawBox	( ControlType -- )			\ added DB 2010-0522
		TypeDrawBox = 
		if   true to GraphicsUsed
		then ;

: DefaultNewControl  ( -- )
                NextControlType ?dup
                if      dup SetType: ThisControl
			dup CheckBitmap			\ CheckBitmap added DB 2009-0501
			dup CheckDrawBox		\ CheckDrawBox added DB 2010-0522
                        dup TypePushButton =
                        if      1 IsOrientation: ThisControl \ default center
                        then    inc-array
                        Initialize: ThisControl
                then    ;

: SetupNewControl       ( -- )
                AddNewControl
		RowColoumsArray? 				\ added for arrays DB 2009-0318
		if   true IsGROUP: ThisControl 
		then Left: FormBox Top: FormBox SetOrigin: ThisControl
		ThatControl 					\ added for Copy-Control DB 2009-0430
		if   Dimensions: ThatControl SetDimensions: ThisControl
		     GetButtonID: ThatControl to ButtonID 
		     SaveButtonID: ThisControl
                     Group?: ThatControl dup IsGroup: ThisControl
                     Global?: ThatControl dup IsGlobal: ThisControl and 
			   if 	GetEdFunction: ThatControl IsEdFunction: ThisControl 
				GetRequirements: ThatControl IsRequirements: ThisControl
			   then
		     GetFunction: ThatControl IsFunction: ThisControl
		     GetBitmap: ThatControl IsBitmap: ThisControl
		else Left: FormBox Top: FormBox 
		     Bottom: FormBox - abs swap Right: FormBox - abs swap 
		     SetDimensions: ThisControl
                then DefaultNewControl ;

: DefaultControlSize   ( -- )  \ set up formbox for default size
                Left: FormBox Top: FormBox
                NextControlType
                case
                     TypeVertScroll             of  25 100      endof
                     TypeFileWindow             of  120 135     endof
                     | TypeMultiLineBox
                       TypeScrollTextBox
                       TypeStaticBitmap
                       TypeListBox
                       TypeGroupBox
                       TypeDrawBox
                       TypeTabControl           |of 120 75      endof
                                                100 25 rot
                endcase 2over d+ SetRect: FormBox ;

: adjust-box    ( --)   \ adjust in case end point is less than beginning point
                Left: FormBox Top: FormBox Right: FormBox Bottom: FormBox
                { l t r b \ w h }
                r l - to w       \ width
                b t - to h       \ height
                w 0<
                if    w +to l
                then  h 0<
                if    h +to t
                then  l w abs + to r
                t h abs + to b
                l t r b SetRect: FormBox ;

:NoName 	( -- )				\ DB 2009-0302
		ButtonID to lastType
		true  IDC_SELECT CheckButton: TheControlToolBar 
                false to newcontrol? ;   is SwitchToSelect


: unclicked     ( -- )
                hwnd Call ReleaseCapture drop
                locked? ?exit
                adjust-box
                newcontrol?
                if      one-click?
			if	false to newcontrol?
			then
                       \ single click will create control with default size
                        Left: FormBox Right: FormBox - abs hsize 3 * <
                        if       DefaultControlSize
                        then	 GetRows GetColoums *		\ added to make arrays
			1 max 256 min 0 			\ 256 DB 2009-0304 
			do SetupNewControl
			Loop	
                        ThisControl IsActiveControl
			SaveButtonID: ActiveControl		\ added to Copy-Paste Objects DB 2009-04-29
                        ShowHandles modified false to selecting?
                then    sizing? moving? or
                if      Left: FormBox Top: FormBox SetOrigin: ActiveControl
                        Right: FormBox Left: FormBox - abs    \ width
                        Bottom: FormBox Top: FormBox - abs   \ height
                        SetDimensions: ActiveControl
                        false to sizing? false to moving? false to selecting?
                        Update: ActiveControl
                        UpdatePropertyWindow
                        ShowHandles modified
                then    false to cclicked?
                Left: FormBox Top: FormBox Right: FormBox Bottom: FormBox d=
                if      false to selecting? NoActiveControl     \ no box drawn
                then    RowColoumsArray? 			\ DB 2009-0315
			ThatControl or				\ DB 2009-0430
				if SwitchToSelect 
				   false to ThatControl		\ DB 2009-0430
				   false to lastType 
				then
			Paint: self     ;

: dosize        { \ deltax deltay -- }
                locked? ?exit
                newcontrol?
                if      Left: FormBox Top: FormBox
                        mousex lastx - Right: FormBox +
                        mousey lasty - Bottom: FormBox +
                        ShowBox exit
                then    hid
                if      hid
                        case
                                1  of sizetopleft     endof
                                2  of sizetopright    endof
                                3  of sizebottomleft  endof
                                4  of sizebottomright endof
                        endcase exit
                then    cclicked?
                if      true to moving?
                        mousex lastx - to deltax mousey lasty - to deltay
                        deltax   Left: FormBox + abs
                        deltay    Top: FormBox + abs
                        deltax  Right: FormBox + abs
                        deltay Bottom: FormBox + abs ShowBox exit
                then    selecting?
                if      Left: FormBox Top: FormBox
                        mousex lastx - Right: FormBox +
                        mousey lasty - Bottom: FormBox +
                        ShowBox
                then    ;
int mousedown
: ?mousepointer (  -- )
                sizing? newcontrol? or moving? or ?exit
                mousedown selecting? and ?exit
                inhandle?
                if      sizeall-cursor
                else    ControlClicked?       \ not really in this case, check if mouse in control
                        if   hand-cursor
                        else arrow-cursor
                        then
                then    ;

:M WM_MOUSEMOVE ( h m w l -- )
                over MK_LBUTTON and to mousedown    \ mouse left button pressed?
                WM_MOUSEMOVE WM: Super
                ?mousepointer
                ;M

:M WM_RBUTTONDOWN  { h m w l -- }
                l set-mousexy
                HideHandles
                ControlClicked? ?dup
                if      locked?
                        if      drop
                        else    IsActiveControl
                        then    ChildPopup to CurrentPopup
                        ?mnu_font
                else    locked? Check: mnu_lock
                        FormPopup  to CurrentPopup
                then    SwitchToSelect   	\ end making childs
			h m w l WM_RBUTTONDOWN WM: Super
                ;M

:M Refresh:     ( -- )
                hwnd 0= ?exitm
                with-statusbar
                if      GetHandle: frmStatusBar 0=
                        if      self Start: frmStatusBar
                                SetSimple: frmStatusBar
                        then
                else    Close: frmStatusBar
                then    ;M

:M Init:        ( -- )
                150 to frmXPos
                175 to frmYPos
                350 to frmWidth
                200 to frmHeight
                0 to controlcount
                s" Form" frmTitle place
                frmTitle +null
                s" Form1" frmName place
                false to Modal
                false to SaveScreen?
\                DIALOG-CLASS to FormSuperClass      \ default to dialog
                SUPER-CLASS to FormSuperClass       \ default to super DB 2012-0407
                false to ChildState
                false to with-statusbar
                false to with-menubar               \ DB 2012-0414   
                false to with-interpreter               \ DB 2012-0511   
                ;M

:M SetName:     ( addr cnt -- )
                31 min 0max frmName place
                ;M

:M ClassInit:   ( -- )
                ClassInit: super
                Erase: upperleft
                Erase: lowerright
                0 to ActiveControl
                0 to ControlList
                0 to modified?
                0 to ThisControl
                0 to sizing?
                0 to moving?
                0 to cclicked?
                0 to selecting?
                true to firsttime?
                false to locked?
                Init: self
                ;M

:M WindowTitle: ( -- )
                frmTitle 1+ ;M

:M StartSize:   ( -- w h )
                0 0                     \ adjust x,y relative to 0,0
                frmWidth frmHeight
                SetRect: TempRect
                0                       \ no menu
                WindowStyle:   [ self ]                 \ the window style
                AddrOf: TempRect                        \ make a new rectangle
                call AdjustWindowRect ?win-error        \ adjust the window
                Right: TempRect Left: TempRect -         \ adjusted  width
                Bottom: TempRect  Top: TempRect -         \ adjusted height
                ;M

:M StartPos:    ( -- x y )
                frmXPos frmYPos ;M

:M WindowStyle: ( -- wstyle )
                WindowStyle: super
                ;M

:M Start:       ( -- )         \ create a new window object
                hWnd 0=
                if      register-frame-window drop
                        create-frame-window to hWnd
                       \  SW_SHOWNORMAL Show: self
                        Update: self
                else    SetFocus: self
                then
                ;M

:M On_Size:     ( -- )
                locked? ?exitm          \ if form is locked disable sizing
                width to frmWidth height to frmHeight
                firsttime? not    \ ignore sizing message when when window is created
                if      modified
                else    false to firsttime?
                then    with-statusbar
                if      Redraw: frmStatusBar
                then    doupdate ;M

:M MinSize:     ( -- w h )  \ if form is locked disable sizing
                locked?
                if      frmwidth frmheight
                else    0 0
                then    ;M

:M Maxsize:     ( -- w h )  \ if form is locked disable sizing
                locked?
                if      frmwidth frmheight
                else    screen-size
                then    ;M

\ Additions to make and execute Batchfiles 	DB 2009 0310, generalized 2009 0609

\ string concatenation:  $1 + $2 -> $1+$2 in pad	\ from apps\Chess\TOOLSET.F	\ added DB 2009 0607
: $concat ( $1 n $2 n - pad n1+2 )
    temp$ place             \ Save old $2. It might be in pad.
    pad   place             \ Put $1 in place.
    temp$ count pad +place  \ Add old $2.
    pad count
 ;

variable id#

: TheBuffer>File ( $ c -- )  \ address and count of filename-string
   w/o create-file                \ adapted from FileToolsHTML.f CreateFile
   abort" Couldn't open file" id# !
   TheBuffer id# @ write-file abort" Couldn't write filebuffer to file"
   id# @ close-file abort" Couldn't close file" \ adapted dito from Filebuffer>File
;

: BatchFileExecute  ( a n -- )   \ needs Batchfilename, saves current Buffer to Batchfile and executes
   2dup s" .bat" $concat TheBuffer>File
   initbuffer
   2dup s"  dos " append append s" .bat " append&crlf	\ generates dos command to execute batch file
   2dup s" .f" $concat TheBuffer>File
   s" .f" $concat INCLUDED
;

: Search2$TheBuffer ( a2 n2 a1 n1 -- flg )	\ true if string a2 is found in TheBuffer after string a1
   TheBuffer 2swap Searchia 
    if   2swap Searchia nip nip
    else 2drop false
    then
;

: SearchTheBuffer ( a n -- flg )	\ true if string is found in TheBuffer
   TheBuffer 2swap Searchia nip nip
;

\ DeleteTurnkeyLoader with Timer to start 	\ DB 2009-0507

: DeleteTurnkeyLoader ( -- )			\ DB 2009-0506
\	cr ." Deleting files"
	s" TurnkeyLoader.f"   delete-file drop
	s" TurnkeyLoader.bat" delete-file drop
	s" SHELL.BAT" delete-file drop
\	cr ." Files deleted"
;

: InitTimer	( -- ) \ init timer for this window to a 4000 ms rate
		#timers to timer \ 1 +to #timers
                0 4000 timer hWnd Call SetTimer drop ;

: ExitTimer	( -- ) \ destroy the timer for this window
                timer hWnd Call KillTimer drop ;

\ End of these additions

:M On_Init:     ( -- )
                ['] dosize      SetTrackFunc: self
                ['] clicked     SetClickFunc: self
                ['] unclicked SetUnClickFunc: self
                FormPopup        SetPopupBar: self   \ need to start
                ChildPopup       SetPopupBar: self   \ both popups
                self to ActiveForm
                array() array-size erase
                FormColor to BackGroundColor
                Refresh: self
                UpdateSystem
                ;M

: drawgrid      ( -- )
                Height: self gridspacing / 1+ 1
                ?do     Width: self gridspacing / 1+ 1
                        ?do     i gridspacing * j gridspacing *
                                GREEN SetPixel: dc
                        loop
                loop    ;

:M On_Paint:    ( -- )
                0 0 Getsize: self FormColor FillArea: dc
                drawgrid
                locked? ?exitm
                ActiveControl 0<> moving? not and sizing? not and
                if      ShowHandles
                then    selecting?
                if      drawdottedbox
                then    ;M

:M On_Done:     ( -- )
     \           DisposeControls
                On_Done: Super
                ;M

:M ParentWindow:        ( -- parent )
                GetHandle: TheMainWindow ;M

:M DeleteControl:      ( -- )
                ActiveControl 0= ?exitm
                GetType: ActiveControl dec-array
                ActiveControl ?ControlNumber >Link#: ControlList
                Free: ActiveControl
                ActiveControl Dispose   \ closes control before disposing
                0 Data!: ControlList
                DeleteLink: ControlList
                NoActiveControl
                Paint: self
                modified ;M

: (DeleteControls:)  { \ ctrl -- }		\ added for convenience DB 2009-0501
            Link#: ControlList >r
            >FirstLink: ControlList
            #controls ?dup 
            	 if 0 do  Data@: ControlList dup to ctrl inbox?
                	if   GetType: ctrl dec-array
       			     ctrl ?ControlNumber >Link#: ControlList
                             Free: ctrl
                             ctrl Dispose  	\ closes control before disposing
                             0 Data!: ControlList
                             DeleteLink: ControlList
                             Update: ctrl modified
                	then >NextLink: ControlList
                      loop    
	    	 then r> >Link#: ControlList
            UpdateStatus: self UpdatePropertyWindow
            Paint: self ;

:M DeleteControls: 	( -- )			\ 2x takes the last one, too DB 2009-0501
		(DeleteControls:)
		(DeleteControls:)
;M

:M ChangeControl: ( -- )
\ not all attributes of the old control are kept
\ the name and some others will have to be re-setup
                  AddNewControl
                  Origin: ActiveControl SetOrigin: ThisControl
                  Dimensions: ActiveControl SetDimensions: ThisControl
                  GetFunction: ActiveControl IsFunction: ThisControl	     \ added DB 2009 0601
                  GetEdFunction: ActiveControl IsEdFunction: ThisControl     \ added DB 2009 0601
                  GetRequirements: ActiveControl IsRequirements: ThisControl \ added DB 2009 0601
                  GetToolTip: ActiveControl IsToolTip: ThisControl	     \ added DB 2009 0601
                  Group?: ActiveControl IsGroup: ThisControl
                  Global?: ActiveControl IsGlobal: ThisControl
                  DefaultNewControl
                  false to newcontrol?  \ clear flag
		  SaveButtonID: ThisControl		\ added to Copy-Paste Objects DB 2009-04-29
                  ButtonID 0 to ButtonID  \ clear buttonid
                  IDC_BITMAP IDC_DRAWBOX between
                  if      false ButtonID   CheckButton: TheControlToolBar
                          true  IDC_SELECT CheckButton: TheControlToolBar
                  then    \ we will keep our position in the tab order
                  ThisControl ?ControlNumber    \ link to be deleted
                  ActiveControl ?ControlNumber dup>r >Link#: ControlList
                  ThisControl Data!: ControlList    \ set new control at active control location
                  >Link#: ControlList       \ return to new link added
                  0 Data!: ControlList      \ zero it
                  DeleteLink: ControlList   \ and delete
                  Free: ActiveControl
                  ActiveControl Dispose     \ ditto for old control
                  r> >Link#: ControlList    \ as we were
                  ThisControl IsActiveControl
                  ShowHandles modified ;M

: (PasteControl:)  ( -- ) 	\ paste Control	\ DB 2009-0429
	  GetType: ThatControl to NextControlType
	  GetButtonID: ThatControl dup IDC_BITMAP IDC_DRAWBOX between
                if      false  IDC_SELECT CheckButton: TheControlToolBar   \ uncheck default
			true   swap   CheckButton: TheControlToolBar   \ select for copy
                       	true to newcontrol?
                then  
;

:M CopyControl: ( -- )      \ copy Control	\ DB 2009-0427
          ActiveControl 0= ?exitm 
          ActiveControl to ThatControl 
	  GetButtonID: ActiveControl to ButtonID
	  SaveButtonID: ThatControl  false to ButtonID
	  (PasteControl:)
;M

:M ChangeToDrawBox: ( -- )
\ not all attributes of the old control are kept
\ the name and some others will have to be re-setup
                  AddNewControl
                  Origin: ActiveControl SetOrigin: ThisControl
                  Dimensions: ActiveControl SetDimensions: ThisControl
                  GetFunction: ActiveControl IsFunction: ThisControl	     \ added DB 2009 0601
                  GetEdFunction: ActiveControl IsEdFunction: ThisControl     \ added DB 2009 0601
                  GetRequirements: ActiveControl IsRequirements: ThisControl \ added DB 2009 0601
                  GetToolTip: ActiveControl IsToolTip: ThisControl	     \ added DB 2009 0601
                  Group?: ActiveControl IsGroup: ThisControl
                  Global?: ActiveControl IsGlobal: ThisControl
                  DefaultNewControl
                  false to newcontrol?  \ clear flag
		  SaveButtonID: ThisControl		\ added to Copy-Paste Objects DB 2009-04-29
                  ButtonID 0 to ButtonID  \ clear buttonid
           \       IDC_BITMAP IDC_DRAWBOX between
           \       if      false ButtonID  cr .s CheckButton: TheControlToolBar 10 spaces .s
           \               true  IDC_SELECT cr .s CheckButton: TheControlToolBar 10 spaces .s
           \       then    \ we will keep our position in the tab order
                  ThisControl ?ControlNumber    \ link to be deleted
                  ActiveControl ?ControlNumber dup>r >Link#: ControlList
                  ThisControl Data!: ControlList    \ set new control at active control location
                  >Link#: ControlList       \ return to new link added
                  0 Data!: ControlList      \ zero it
                  DeleteLink: ControlList   \ and delete
                  Free: ActiveControl
                  ActiveControl Dispose     \ ditto for old control
                  r> >Link#: ControlList    \ as we were
                  ThisControl IsActiveControl
                  ShowHandles modified ;M

:M ChangeToScrollTextBox: ( -- )
\ not all attributes of the old control are kept
\ the name and some others will have to be re-setup
                  AddNewControl
                  Origin: ActiveControl SetOrigin: ThisControl
                  Dimensions: ActiveControl SetDimensions: ThisControl
                  GetFunction: ActiveControl IsFunction: ThisControl	     \ added DB 2009 0601
                  GetEdFunction: ActiveControl IsEdFunction: ThisControl     \ added DB 2009 0601
                  GetRequirements: ActiveControl IsRequirements: ThisControl \ added DB 2009 0601
                  GetToolTip: ActiveControl IsToolTip: ThisControl	     \ added DB 2009 0601
                  Group?: ActiveControl IsGroup: ThisControl
                  Global?: ActiveControl IsGlobal: ThisControl
                  DefaultNewControl
                  false to newcontrol?  \ clear flag
		  SaveButtonID: ThisControl		\ added to Copy-Paste Objects DB 2009-04-29
                  ButtonID 0 to ButtonID  \ clear buttonid
                  ThisControl ?ControlNumber    \ link to be deleted
                  ActiveControl ?ControlNumber dup>r >Link#: ControlList
                  ThisControl Data!: ControlList    \ set new control at active control location
                  >Link#: ControlList       \ return to new link added
                  0 Data!: ControlList      \ zero it
                  DeleteLink: ControlList   \ and delete
                  Free: ActiveControl
                  ActiveControl Dispose     \ ditto for old control
                  r> >Link#: ControlList    \ as we were
                  ThisControl IsActiveControl
                  ShowHandles modified ;M

:M MoveToFront:  ( -- )  \ position control in list so it will be found first
                ActiveControl 0= ?exitm
                #controls 2 < ?exitm
                ActiveControl ?controlnumber
                >LastLink: ControlList Link#: ControlList over =
                if      drop exitm       then             \ already in front
                    >Link#: ControlList
                     Data@: ControlList >r
                DeleteLink: ControlList
                   AddLink: ControlList
                r>   Data!: ControlList
                     Paint: self
                     modified ;M

:M MoveToBack:   ( -- )  \ position control in list so it will be found last
                ActiveControl 0= ?exitm
                #controls 2 < ?exitm
                ActiveControl ?controlnumber
                >FirstLink: ControlList Link#: ControlList over =
                if      drop exitm       then              \ already at back
                    >Link#: ControlList
                     Data@: ControlList >r
                DeleteLink: ControlList
                >FirstLink: ControlList
                InsertLink: ControlList
                r>   Data!: ControlList
                     Paint: self
                     modified ;M

:M Locked?:     ( -- f )
                locked? ;M

:M Lock:        ( -- )
                locked? 0= dup to locked?           \ toggle flag
                0=
                if      frmtitle count SetText: self
                        exitm
                then    false to modified?
                ShowModified
                HideHandles
                NoActiveControl
                frmTitle count pad place
                s" (Locked)" pad +place
                pad count Settext: self
                false to selecting?
                Paint: self ;M

: IsNewForm?    ( -- f )  \ has form not yet been saved?
                formname count s" untitled.ff" istr= ;

: SaveIt?       { \ temp$ -- f }
                join$( 	s" File "
			formname count
			s"  has been modified. Save it?"
                )join$  1+
                z" Heads up!" MB_YESNO MB_ICONQUESTION or
                MessageBox: self IDYES = ;

: ShowFileName  { \ temp$ -- }
                join$(  ForthFormTitle$
			s"  - "
			formname count
                )join$  count SetText: TheMainWindow ;

: ?addext       ( -- )
                formname count ".ext-only" nip 0=
                if      s" .ff" formname +place
                then    formname +null ;

: SaveForm      (  --  )
                modified? 0= ?exit
                IsNewForm?
                if      hwnd Start: SaveFormDlg
                        dup c@ 0=
                        if      drop exit
                        then    count formname place ?addext
                        ShowFileName
                then    formname count SetName: FormFile
                Create: FormFile ?exit
                #controls SetCount: self
                GetData: self Write: FormFile   \ save header first
                if      Close: FormFile exit    \ exit if error
                then    ControlList
                if      >FirstLink: ControlList         \ next the control info
                        #controls 0
                        ?do     Data@: ControlList to ThisControl
                                GetData: ThisControl Write: FormFile
                                if      Close: FormFile unloop  \ error ocurred
                                        exit
                                then    >NextLink: ControlList
                        loop
                then    Close: FormFile false to modified? ;

:M Save:        ( -- )
                SaveForm ;M

:M SaveAs:        ( -- )
                hwnd Start: SaveFormDlg dup c@
                if      count formname place ?addext ShowFileName
                        modified
                        SaveForm
                else    drop
                then    ;M

:M SetFileName: ( addr cnt -- )
                formname place ;M

:M GetFileName: ( -- addr cnt )
                formname count ;M

: LoadForm      ( -- )
                formname count SetName: FormFile
                Open: FormFile ?exit
                GetData: self Read: FormFile
                if      Close: FormFile
                        exit
                then    GetCount: self 0
                ?do     AddNewControl
                        GetData: ThisControl Read: FormFile
                        if      leave       \ don't exit
                        else    DoInit: ThisControl
                                GetType: ThisControl dup CheckBitmap dup CheckDrawBox inc-array  \ Checks added DB 2009/2010
                        then
                loop    Close: FormFile
                ShowFileName
                frmtitle count SetTitle: Self
                StartSize: self hwnd AdjustWindowSize
                Refresh: self ;

:M Load:        ( -- f )
                LoadForm
                ;M

:M On_SetFocus: ( -- )
                self to ActiveForm
                ShowFileName
                doUpdate
                UpdateStatus: self
                ActiveControl
                if      UpdatePropertyWindow
                then    UpdateSystem ;M

:M Close:       ( -- )
                modified?
                if      SaveIt?
                        if      SaveForm
                        then
                then   DisposeControls
		Close: Super ;M

:M WM_CLOSE     ( -- ) \ let parent do the closing
                self doCloseForm
                ;M

:M PushKey:     ( c -- )
                case
     ( Ctrl-R )         'R' +k_control  of      doSaveAs        endof
     ( Ctrl-S )         'S' +k_control  of      doSave          endof
     ( Ctrl-T )         'T' +k_control  of      doTest          endof
     ( Ctrl-V )         'V' +k_control  of      doEditor        endof
     ( Ctrl-W )         'W' +k_control  of      doWrite         endof
                         K_DELETE       of      dodelete        endof
                         K_LEFT         of      adjust-left     endof
                         K_RIGHT        of      adjust-right    endof
                         K_UP           of      adjust-up       endof
                         K_DOWN         of      adjust-down     endof
                         K_F12          of  K_F12 PushKey: TheMainWindow endof
                         K_F9           of   K_F9 PushKey: TheMainWindow endof
                         K_F1           of   K_F1 PushKey: TheMainWindow endof
                endcase ;M

:M Display:     ( -- )
                SW_SHOWNORMAL Show: self
                ;M

:M ControlList:        ( -- lit )
                ControlList
                ;M

:M ActiveControl:       ( -- ctrl )
                ActiveControl
                ;M

:M SetActiveControl: ( ctrl -- )
        to ActiveControl
        ;M

:M #Controls:   ( -- n )
                #controls
                ;M

:M IsModified:  ( -- )
                modified
                ;M

:M FormName:    ( -- addr )
                frmName
                ;M

:M IsFormName:  ( addr cnt -- )
                SetName: self
                ;M

:M FormTitle:   ( -- addr )
                frmTitle
                ;M

:M IsFormTitle: ( addr cnt -- )
                maxstring 102 - min 0max frmTitle place \ changed from 2 - to 102 - DB 2012-0414
                frmtitle +null
                ;M

\ Added for MenuBar             \ DB 2012-0414

:M FormMenuBar:   ( -- addr )
                frmMenuBar
                ;M

:M IsFormMenuBar: ( addr cnt -- )
                100 min 0max frmMenuBar place
                frmMenuBar +null
                ;M

\ Additions to add Code 	\ DB 2009 0524-0527

:M FormNeedsFunction:   ( -- addr )
                frmNeedsFunction
                ;M

:M IsFormNeedsFunction: ( addr cnt -- )
                FormNeedsMax 2 - min 0max frmNeedsFunction place
                frmNeedsFunction +null
                ;M

:M FormInitFunction:   ( -- addr )
                frmInitFunction
                ;M

:M IsFormInitFunction: ( addr cnt -- )
                FormFunctionMax 2 - min 0max frmInitFunction place
                frmInitFunction +null
                ;M

:M FormInitializedFunction:   ( -- addr )
                frmCloseFunction
                ;M
 
:M IsFormInitializedFunction: ( addr cnt -- )
                FormFunctionMax 2 - min 0max frmCloseFunction place
                frmCloseFunction +null
                ;M

:M FormPaintFunction:   ( -- addr )         \ added DB 2012-0428
                frmPaintFunction
                ;M

:M IsFormPaintFunction: ( addr cnt -- )     \ added DB 2012-0428
                FormFunctionMax 2 - min 0max frmPaintFunction place
                frmPaintFunction +null
                ;M

:M FormSizeFunction:   ( -- addr )          \ added DB 2012-0428
                frmSizeFunction
                ;M

:M IsFormSizeFunction: ( addr cnt -- )      \ added DB 2012-0428
                FormFunctionMax 2 - min 0max frmSizeFunction place
                frmSizeFunction +null
                ;M

:M FormDoneFunction:   ( -- addr )
                frmDoneFunction
                ;M

:M IsFormDoneFunction: ( addr cnt -- )
                FormFunctionMax 2 - min 0max frmDoneFunction place
                frmDoneFunction +null
                ;M

\ End of theses Additions

:M Origin:      ( -- x y )
                frmXPos frmYPos
                ;M

:M SetOrigin:   ( x y -- )
                to frmYPos to frmXPos
                ;M

:M Dimensions:  ( -- w h )
                frmWidth frmHeight
                ;M

:M SetDimensions: ( w h -- )
                to frmHeight to frmWidth
                ;M

:M SetModal:    ( n -- ) \ set modal flag
                to modal ;M

:M GetModal:    ( -- n )
                modal ;M

:M IsSaveScreen?: ( f -- )
        to SaveScreen? ;M

:M SaveScreen?: ( --f )
        SaveScreen? ;M

:M GetSuperClass: ( -- n )
        FormSuperClass ;M

:M IsSuperClass: ( n -- )
        to FormSuperClass ;M

:M ChildState:  ( -- f )
        ChildState ;M

:M IsChildState: ( f -- )
        to ChildState ;M

:M MenuBar?:  ( -- f )
        with-menubar ;M

:M IsMenuBar: ( f -- )
            to with-menubar ;M

:M StatusBar?:  ( -- f )
        with-statusbar ;M

:M IsStatusBar: ( f -- )
            to with-statusbar ;M

:M Interpreter?:  ( -- f )
        with-interpreter ;M

:M IsInterpreter: ( f -- )
            to with-interpreter ;M


\ ****************** Definitions to write form information ********************

: writeGroupStyle   ( -- )      \ write WS_GROUP style if needed
                 Group?: ThisControl
                GetType: ThisControl TypeRadioButton <> and
                if      +crlf 2tabs s" WS_GROUP +Style: " append
                        GetName: ThisControl append
                then    ;



: writecommoncode   ( -- )     \ startup code common to all controls
                +crlf
                2tabs s" self Start: " append GetName: ThisControl append&crlf
                2tabs Origin: ThisControl swap
		#append zooming @
		if s"  zoom" append frmName count append s"  " append then 	\ DB 090228 form zoom
		#append zooming @
		if s"  zoom" append frmName count append s"  " append then	\ DB 090228 form zoom
                Dimensions: ThisControl swap 
		#append zooming @ 
		if s"  zoom" append frmName count append s"  " append then 	\ DB 090228 form zoom
		#append zooming @
		if s"  zoom" append frmName count append s"  " append then	\ DB 090228 form zoom
                GetType: ThisControl TypeDrawBox =
                if s" 2dup to screen-height to screen-width " append then	\ DB 2010-0522 drawbox
                s" Move: " append GetName: ThisControl append
                writeGroupStyle ;

: writetext     ( -- ) \ code to set the text of control
                +crlf
                2tabs s"append GetTitle: ThisControl append "append
                s"  SetText: " append GetName: ThisControl append  ;

: writeStyle    ( addr cnt -- )
                +crlf
                2tabs append s"  +Style: " append GetName: ThisControl append ;

: fontname      ( -- name cnt )
                GetName: ThisControl pad place s" -font" pad +place pad count ;

variable zoomdefDone
variable zoomfont	\ added to mark use of ZoomFont DB 2009-1120


: writefont     ( -- )
                +crlf 2tabs
                FontChanged: ThisControl
                if     s" Set-" append fontname append&crlf 2tabs
                       s" Create: " append fontname append&crlf 2tabs
                       s" Handle: " append fontname append s"  SetFont: " append GetName: ThisControl append
                else   zooming @
		       if   s" Set-Zoom-font" append&crlf 2tabs
                            zoomdefDone @ 2 < if s" Create: Zoom-font" append&crlf 2tabs 2 zoomdefDone ! then
		            s" Handle: Zoom-font SetFont: " append GetName: ThisControl append
		       else s" Handle: Winfont SetFont: " append GetName: ThisControl append
                       then
                then   ;

-11 value FormFont



: write-zoomfont   ( -- )  \ create a definition for a zoomable font			   \ DB 090228
   +crlf 
   s" Font Zoom-font" append&crlf
   s" : Set-Zoom-font ( -- )" append&crlf 2tabs
   FormFont #append s"  zoom" append frmName count append s"  Height: Zoom-font" append&crlf 2tabs
   s" 0             Width: Zoom-font" append&crlf 2tabs
   s" 0        Escapement: Zoom-font" append&crlf 2tabs
   s" 0       Orientation: Zoom-font" append&crlf 2tabs
   s" 400          Weight: Zoom-font" append&crlf 2tabs
   s" 0           CharSet: Zoom-font" append&crlf 2tabs
   s" 3      OutPrecision: Zoom-font" append&crlf 2tabs
   s" 2     ClipPrecision: Zoom-font" append&crlf 2tabs
   s" 1           Quality: Zoom-font" append&crlf 2tabs
   s" 34   PitchAndFamily: Zoom-font" append&crlf 2tabs
   ascii s cappend ascii " cappend s"  Arial" append ascii "  cappend s"  SetFacename: Zoom-font ;" append&crlf
   zoomfont on					\ mark that a zoomfont has been written	   \ DB 091120	
;

: write-fontfuncs  ( -- )  \ create a definition for each font to be changed and zoom-font \ DB 090228
                +crlf
                s" \ Font setting definitions" append&crlf	\   put on top from below  \ DB 090228
                #controls 0= ?exit
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        FontChanged: ThisControl
                        if          +crlf
                                    i 1 =  \ comment once only
                                    if     \ s" \ Font setting definitions" append&crlf
                                    then   s" Font " append fontname append&crlf
                                     s" : Set-" append fontname append s"  ( -- )" append&crlf
                                     2 +tabs FontData: ThisControl drop
                                     dup @ (.)  5 append.l zooming @
                      				if   s"  zoom" append frmName count append s"  Height: "
						else s"         Height: "
						then append fontname append&crlf 2tabs cell+
                                     dup @ (.)  5 append.l s"          Width: " append fontname append&crlf 2tabs cell+
                                     dup @ (.)  5 append.l s"     Escapement: " append fontname append&crlf 2tabs cell+
                                     dup @ (.)  5 append.l s"    Orientation: " append fontname append&crlf 2tabs cell+
                                     dup @ (.)  5 append.l s"         Weight: " append fontname append&crlf 2tabs cell+
                                     dup c@ if             s" true    Italic: " append fontname append&crlf 2tabs then 1+
                                     dup c@ if             s" true Underline: " append fontname append&crlf 2tabs then 1+
                                     dup c@ if             s" true StrikeOut: " append fontname append&crlf 2tabs then 1+
                                     dup c@ (.) 5 append.l s"        CharSet: " append fontname append&crlf 2tabs 1+
                                     dup c@ (.) 5 append.l s"   OutPrecision: " append fontname append&crlf 2tabs 1+
                                     dup c@ (.) 5 append.l s"  ClipPrecision: " append fontname append&crlf 2tabs 1+
                                     dup c@ (.) 5 append.l s"        Quality: " append fontname append&crlf 2tabs 1+
                                     dup c@ (.) 5 append.l s" PitchAndFamily: " append fontname append&crlf 2tabs 1+
                                     s"append zcount append "append s"  SetFacename: " append fontname append s"  ;" append&crlf
			else 	     zooming @ if  zoomdefDone @ 0=  if write-zoomfont 1 zoomdefDone ! then then
                        then
                loop    ;

: write-delete-fonts ( -- )
                #controls 0= ?exit
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        FontChanged: ThisControl
                        if     2tabs s" Delete: " append fontname append&crlf
                        then
                loop    zoomfont @ 		\ changed from zooming to zoomfont	\ DB 2009-11-20
		if      2tabs s" Delete: Zoom-font" append&crlf
		then ;
comment:
: writeMenuBar  ( -- )  \ MenuBar for TurnKey DB 2009-0307
		s" MENUBAR ApplicationBar" append&crlf
		2tabs s" POPUP " append "append s" File" append "append +crlf
		2tabs s" MENUITEM        " append "append s" Exit" append "append
		s"       Close: " append frmName count append s"   bye  ;" append&crlf
\ 
		2tabs s" POPUP " append "append s" Help" append "append +crlf
		2tabs s" MENUITEM        " append "append s" Info" append "append +crlf
		2tabs 2tabs z"append s"  Info" append "append +crlf
		2tabs 2tabs z"append s"  A template for a minimal window." append "append +crlf
		2tabs 2tabs s" msgBox: " append frmName count append s"  ;" append&crlf
\
		s" ENDBAR" append&crlf
;
comment;

: (WriteBitmapDef) ( a n -- )  		\ write bitmap ( a n -- ) definition	\ extracted from WriteBitmapDefs DB 2009 0702
		2dup s" 0 value " 2swap 4 - $concat 
		2dup SearchTheBuffer 0=
			if   append s"   \ is Bitmap handle for " append 2dup append&crlf 2dup >Bitmapbuffer
			else 2drop
			then	
		2dup s" : Load:" 2swap 4 - $concat 
		2dup SearchTheBuffer 0=
			if   append BL cappend z"append 2dup append ( bitmap ) "append 
		     	     s"  LoadBitmap to " append 4 - append s"  ;" append&crlf
			else 2drop
			then
;

0 value BitmapState

: WriteGlobalBitmapDefs { \ ctrl -- }  \ write global bitmap definitions	\ renamed from WriteBitmapDefs DB 2009 0702
                #controls 0= ?exit
		Erase-Bitmapbuffer	\ Prepare collection of Bitmaps DB 2009 0729
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        Global?: ThisControl
			GetType: ThisControl TypeBitmapButton =
			GetType: ThisControl TypeStaticBitmap = or and
			   if	 GetBitmap: ThisControl dup 0=			 \ added DB 2009 0605
					if   2drop s" vF.bmp" 
					then BitmapState true =
						if   2dup SearchTheBuffer 0=
							if   append&crlf 
							else 2drop
							then
						else (WriteBitmapDef)
						then
			   then
                loop 
;

\ additions for Bitmaps DB 2009 0702

: GetFormNeeds  ( -- a n / flg )	\ Gets part of "This Form needs:" which may contain Bitmaps or files needed
			\ flg is false if there is none
		FormNeedsFunction: self count ?dup	\ copied from "writeThisFormNeeds"
		if   2dup s" needs" searchia
			if  nip - else 2drop then
                     2dup s" 0 value" searchia		\ if value comes first, has to be 0 !
			if  nip - else 2drop then
                     2dup s" variable" searchia
			if  nip - else 2drop then
                     2dup s" noop" searchia		\ noop as delimiter afte Bitmaps and files, optional usable
			if  nip - else 2drop then
                     2dup s" \" searchia		\ Backslash as delimiter afte Bitmaps and files, optional usable
			if  nip - else 2drop then
                     2dup s" : " searchia
			if  nip - else 2drop then
		     dup 1 < if 2drop false then	\ Security - should give an errormessage !
		else drop false
		then
;

: Forthfile? 	  ( a n -- flg )
 		dup 2 > if   2 - + 2 s" .f"  ISTR= 
			else 2drop false
			then
;


: Formfile? 	  ( a n -- flg )
 		dup 4 > if   4 - + 4 s" .frm"  ISTR= 
			else 2drop false
			then
;

: NeededFile? 	  ( a n -- flg )
		2dup Forthfile? >r Formfile? r> or
;

: Bitmapfile? 	  ( a n -- flg )
 		dup 4 > if   4 - + 4 s" .bmp"  ISTR= 
			else 2drop false
			then
;
comment:
: Skip-BL	  ( a n -- a n )	\ skips leading BL's in String
		dup 0> if
		2dup bounds do i c@ BL <= if 1- swap 1+ swap else leave then loop 
		       then
;

: GetWord	  ( a n -- a n )	\ gets first word in string
		dup 0> if
		Skip-BL over -rot 0 -rot bounds 
                         do i c@ BL <= if leave else 1+ then loop 
		       then
;

102 value BMP-ID
comment;					\ DB 2009 0609

: (Load:Bitmap)	( a n -- )	\ extracted from (Load:Bitmaps) 	\ DB 2009 0707
		dup
		if   4 - ( bitmap ) 
		else 2drop s" vF"
		then turnkey?
		if   s"  AppInst call LoadBitmap to " 						     
                else s" Load:"
		then 2swap $concat
		2dup s" \ FormInit-Functions:" Search2$TheBuffer 0=
		if   2tabs turnkey?
			if   BMP-ID dup (.) append 1+ to BMP-ID
			then append&crlf 
		else 2drop
		then 	
; 

: (WriteDeleteBitmap)	( a n -- )
			dup	if   4 - ( bitmap ) 
				else 2drop s" vF"
				then s"  call DeleteObject drop" $concat 
				2dup SearchTheBuffer 0=
				if   2tabs append&crlf 
				else 2drop
				then 	
;

: WriteBitmapCase ( a n -- )
	BitmapState case 1 of (WriteBitmapDef)		endof
			 2 of (Load:Bitmap)		endof
			 3 of (WriteDeleteBitmap)	endof
			 2drop
		 endcase
;

: WriteNeededFile ( a n -- )
	BitmapState  0= if s" needs " append append&crlf
			else 2drop
			then
;

: WriteFormCases ( -- )  		\ write definitions for "This Form needs:" Bitmaps 
		GetFormNeeds ?dup 0= ?exit
		2dup bounds do i over GetWord ?dup 0= if leave then dup -rot 2dup Bitmapfile?
			if   WriteBitmapCase 
			else 2dup NeededFile?
				if   WriteNeededFile
				else 2drop
				then
			then dup >r - r> +loop 2drop
;

\ end additions DB 2009 0702

: AddResources ( -- )				\ to add resources 	DB 2009 0609
	102 to BMP-ID
	&forthdir count pad place
	frmName count pad +place s" .exe" pad +place
	pad count 2dup file-status nip 0=
	    if	 "path-file drop AddToFile
		 AddBitmaps
		 false EndUpdate
	    else 2drop true s" EXE-File not found " ?MessageBox 
	    then
;

:M WM_TIMER     ( h m w l -- res )	\ handle the WM_TIMER events \ DB 2009 0609
                ExitTimer 		\ destroy the timer
		AddResources
		DeleteTurnkeyLoader
                Paint: self		\ refresh the window
                0 ;M


: writeneeds    ( -- )  \ what files are needed by the form
                s" \ " append TextFile: [ self ] "to-pathend" 2dup upper append&crlf
                s" anew " append TextFile: [ self ] "to-pathend" 2dup upper append&crlf
		s" Decimal" append&crlf
                #controls 0<> with-statusbar or
                if      s" \- textbox needs excontrols.f" append&crlf
\ only add "needs bitmap.f" if we have a bitmap or an icon button
                        BitmapUsed					\ changed for convenience DB 2009-0501
                        if      s" \- usebitmap needs bitmap.f" append&crlf
				WriteGlobalBitmapDefs
				1 to BitmapState	\ added DB 2009 0707
				WriteFormCases		\ added DB 2009 0702
                        then
                        GraphicsUsed					\ added DB 2010-0522
                        if      s" \- moveto needs GraphicTools.f" append&crlf
                        then
                        TypeFileWindow  get-array       \ do we have any?
                        if      s" \- -filelister.f needs filelister.f   \ folder browser" append&crlf
                        then
                then    GetSuperClass: self MDIDIALOG-CLASS =
                if   s" \- notcancelled needs mdi.f         \ multiple document interface routines" append&crlf
                     s" \- mdidialogwindow needs mdidialog.f    \ actual super class" append&crlf
                then 
		+crlf
	zooming @							      \ DB 2009-0409 make zoom-functions invisible
	if	s" \- zoom 100 value Zoom% : zoom Zoom% 100 */ ;" append&crlf \ DB 090228 general zoom - from writeopening DB 2009 0601
                +crlf							      \ "defined" changed to \-	DB 2009-0405
\		s" 100 value Zoom%" append frmName count append&crlf
\		s" 60 value Zoom%" append frmName count append&crlf           \ DB 2013-0729
		ZoomDefault (.) append s"  value Zoom%" append frmName count append&crlf  \ DB 2013-0815
		+crlf
		s"  : zoom" append frmName count append 
		s"  Zoom%" append frmName count append s"  100 */ zoom ;" append&crlf \ DB 090228 form zoom
		+crlf
	then	s" \- Control# 0 value Control#" append&crlf 	 \ DB 090317 for arrays	\ should belong to fkernel	DB 2009 0601
;

: writeThisFormNeeds    ( -- )  \ Words added		\ placed after Bitmap	DB 2009-0622
		FormNeedsFunction: self count ?dup	\ added DB 2009-0527
		if   GetFormNeeds ?dup		\ added DB 2009 0702
			if dup >r + rot drop swap r> - then 	\ add only part after Bitmaps and files
		     ?dup if   +crlf s" \ This Form needs:" append&crlf +crlf 
			       append&crlf +crlf 
			  else drop 
			  then 
		else drop
		then MenuBar?: self
                        if   s" needs " append
                             FormMenuBar: self count append&crlf +crlf 
                        then	
;

: writeopening  ( -- )
		+crlf
                s" :Object " append frmName count append
                2tabs s" <Super " append
                GetSuperClass: self dup CHILD-CLASS =
                if      drop s" Child-Window"
                else    dup MDIDIALOG-CLASS =
                        if   drop s" MDIDialogWindow"
                        else SUPER-CLASS =        \ added  DB 2013-0702
                              if   s" Window"     \ added  DB 2013-0702
                              else s" DialogWindow"
                        then  then
                then    append&crlf
                +crlf s" Font WinFont           \ default font" append&crlf     \ font for controls
                s" ' 2drop value WmCommand-Func   \ function pointer for WM_COMMAND" append&crlf
                s" ColorObject FrmColor      \ the background color " append&crlf
                with-statusbar
                if      with-multistatusbar @
                        if   s" MultiStatusbar " 
                        else s" StatusBar "
                        then append
                        s" TheStatusBar" append&crlf
                then    GetSuperclass: self DIALOG-CLASS =      \ only for dialogwindow super class
                if  SaveScreen?
                    if  frmXPos #append frmYPos #append
                        s"  2value XYPos  \ save screen location of form" append&crlf
                    then
                then    write-fontfuncs ;

: write-child-params    { \ cx cy cw ch -- }
\ For a child window we want only the size and position
                Origin: ThisControl to cy to cx
                Dimensions: ThisControl to ch to cw
                s" \ Coordinates and dimensions for " append GetName: ThisControl append&crlf
                cx #append s"  value " append GetName: ThisControl append s" X" append&crlf
                cy #append s"  value " append GetName: ThisControl append s" Y" append&crlf
                cw #append s"  value " append GetName: ThisControl append s" W" append&crlf
                ch #append s"  value " append GetName: ThisControl append s" H" append&crlf
                ;

: writeglobals  ( -- )  \ write any global controls
                #controls 0= ?exit
                +crlf   \ force start on new line
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        Global?: ThisControl
                        if      GetType: ThisControl
                                case
                                        TypeTextBox        of Spinner?: ThisControl
                                                              if   s" SpinnerControl "
                                                              else s" TextBox "
                                                              then                  endof
                                        TypeGroupBox       of s" GroupBox "         endof
                                        TypePushButton     of s" PushButton "       endof
                                        TypeCheckBox       of s" CheckBox "         endof
                                        TypeRadioButton    of Group?: ThisControl
                                                              if   s" GroupRadioButton "
                                                              else s" RadioButton "
                                                              then                  endof
                                        TypeBitmapButton   of s" BitmapButton "     endof
                                        TypeListBox        of s" ListBox "          endof
                                        TypeComboBox       of s" ComboBox "         endof
                                        TypeHorizScroll    of s" HorizScroll "      endof
                                        TypeVertScroll     of s" VertScroll "       endof
                                        TypeLabel          of s" Label "            endof
                                        TypeComboListBox   of s" ComboListBox "     endof
                                        TypeMultiLineBox   of s" MultiLineTextBox " endof
                                        TypeStaticBitmap   of s" StaticBitmap "     endof
                                        TypeMultiListBox   of s" MultiListBox "     endof
                                        TypeFileWindow     of s" FileWindow "       endof
                                        TypeTabControl     of s" TabControl "       endof
                                        TypeDrawBox        of s" DrawBox "          endof
                                        TypeScrollTextBox  of s" ScrollTextBox "    endof
                                        TypeGeneric        of write-child-params
                                                              s" "                  endof
                                        s"  \ UnKnown Control " ( default ) rot
                                endcase ?dup
                                if      append GetName: ThisControl append&crlf
                                else    drop
                                then
                        then
                loop    ;

: ?BLonly   ( addr count -- flag )  \ true if empty or only Blanks
                ?dup    if   0 do dup I + C@ BL > if drop 0 leave then loop
                        then ;

: writeRequirements  ( -- )  \ write this Function's Requirements
    \ to type in Requirements for execution ( variables, values, functions, numbers, etc. 2008-07-15 DB
                GetFunction: ThisControl ?BLonly 0=
                GetRequirements: ThisControl ?BLonly 0= and
                        if   GetRequirements: ThisControl 
			     2dup SearchTheBuffer 0=				    \ added DB 2009 0611
					if   append&crlf 
					else 2drop
					then +crlf
                        then ;


: GetObject# ( adress len -- adress len )     	\ gets Number after last alpha char \ added DB 2009-0317
   2dup >r dup r> 0 do over i + c@ ascii 0 ascii 9 1+ within 0=
			if drop dup i + 1+ then loop nip
   dup >r rot - ( difference between old and new address ) - ( new length )
   r> swap dup 0= if nip (.) then
;

variable LocalFunction? 

: writeLocalFunction  ( -- )  \ write this local Function		\ DB 2009-1121
                GetFunction: ThisControl 2dup ?BLonly 0=
                GetEdFunction: ThisControl ?BLonly 0= and
		GROUP?: ThisControl or					\ added for arrays DB 2009-0318
                        if   s" : " append ?dup 0= 			\ dito
			     if   drop s" SetControl#" then append	\ dito
			     GROUP?: ThisControl
			     if   GetName: ThisControl GetObject# append	\ dito
			     then s"   ( -- ) " append&crlf
 	  		     GROUP?: ThisControl 			\ added for arrays DB 2009-0318
			     if	  GetName: ThisControl GetObject# 	\ added DB 2009-0317
			     	  append s"  to Control#" append&crlf	\ added DB 2009-0317
                             then GetEdFunction: ThisControl dup 0=
					if   GROUP?: ThisControl	\ added DB 2009 0611
						if   2drop GetFunction: ThisControl then
			 	   	then append&crlf
                             S" ; "  append&crlf  \ +crlf		\ removed DB 2009-0409 
                        then s" : On_" ;

: writeLocalFunctions  ( -- )  \ write any local Functions	\ DB 091121
                #controls 0= ?exit
                +crlf   \ force start on new line
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        Global?: ThisControl 0=
                        if      GetType: ThisControl
                                case    TypePushButton     of writeRequirements
                                                              writeLocalFunction endof
                                        TypeBitmapButton   of writeRequirements
                                                              writeLocalFunction endof
                                        TypeGroupBox       \ of endof
                                         s"  \ Not executable: " ( default ) rot
                                endcase ?dup
                                if      LocalFunction? on
                                        append GetName: ThisControl 
					append s"   ( -- ) " append \ &crlf removed DB 2010-0502
                                        GetFunction: ThisControl 2dup ?BLonly 
					if  2drop GROUP?: ThisControl
						if   s" SetControl#" 
						else s"  noop" 
                                        	then 
					then append GROUP?: ThisControl
					if   GetName: ThisControl GetObject# append
                                       	then ( +crlf removed DB 2010-0502 ) s"  ;" append&crlf
                                        +crlf
                                else    drop
                                then
                        then
                loop    ;

: SetDefaultCommand  ( -- )  \ set control DefaultCommand	\ DB 091121
                2tabs s"     GetID: " append GetName: ThisControl 2dup append
                s"   of On_" append append s"  endof" append&crlf 
;

: writeDefaultCommand  ( -- )  \ write form DefaultCommand	\ DB 091121
                +crlf   \ force start on new line
                s" : frmDefaultCommand	( id obj-addr -- )" append&crlf
                2tabs s" swap" append&crlf 2tabs s" case" append&crlf
                #controls 0= ?exit
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        Global?: ThisControl 0=
                        if      GetType: ThisControl
                                case    TypePushButton     of SetDefaultCommand endof
                                        TypeBitmapButton   of SetDefaultCommand endof
                                endcase 
                        then
                loop    2tabs s" endcase   drop ;" append&crlf
;

: writeDefaultCommandLink  ( -- )  \ write link to DefaultCommand	\ DB 091121
                +crlf   \ force start on new line
                2tabs s" ['] frmDefaultCommand SetCommand: self" append&crlf
;
                

: writeGlobalFunction  ( -- )  \ write this global Function
                          \ to type in a word for execution  2008-07-08 DB
                GetFunction: ThisControl 2dup ?BLonly 0=
                GetEdFunction: ThisControl ?BLonly 0= and
		GROUP?: ThisControl or					\ added for arrays DB 2009-0318
                        if   s" : " append ?dup 0= 			\ dito
			     if   drop s" SetControl#" then append	\ dito
			     GROUP?: ThisControl
			     if   GetName: ThisControl GetObject# append	\ dito
			     then s"   ( -- ) " append&crlf
 	  		     GROUP?: ThisControl 			\ added for arrays DB 2009-0318
			     if	  GetName: ThisControl GetObject# 	\ added DB 2009-0317
			     	  append s"  to Control#" append&crlf	\ added DB 2009-0317
                             then GetEdFunction: ThisControl dup 0=
					if   GROUP?: ThisControl	\ added DB 2009 0611
						if   2drop GetFunction: ThisControl then
			 	   	then append&crlf
                             S" ; "  append&crlf  \ +crlf		\ removed DB 2009-0409 
                        then s" ' " ;

: writeFunctions  ( -- )  \ write any global Functions
                          \ to type in a word for execution  2008-07-08 DB
                #controls 0= ?exit
                +crlf   \ force start on new line
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        Global?: ThisControl
                        if      GetType: ThisControl
                                case    TypePushButton     of writeRequirements
                                                              writeGlobalFunction endof
                                        TypeBitmapButton   of writeRequirements
                                                              writeGlobalFunction endof
				comment:
				\ DB 2009-04-08 added writeRequirements 
				\ to the following in writeGlobalRequirements,
				\ if any of the following is added, it has to be removed 
				\ from writeGlobalRequirements below.      
                                        TypeCheckBox       of writeRequirements
                                                              writeGlobalFunction endof
                                        TypeRadioButton    of writeRequirements
                                                              writeGlobalFunction endof
                                        TypeBitmapButton   of writeRequirements
                                                              writeGlobalFunction endof
                                        TypeListBox        of writeRequirements
                                                              writeGlobalFunction endof
                                        TypeComboBox       of writeRequirements
                                                              writeGlobalFunction endof
                                        TypeHorizScroll    of writeRequirements
                                                              writeGlobalFunction endof
                                        TypeVertScroll     of writeRequirements
                                                              writeGlobalFunction endof
                                        TypeComboListBox   of writeRequirements
                                                              writeGlobalFunction endof
                                        TypeStaticBitmap   of writeRequirements
                                                              writeGlobalFunction endof
                                        TypeMultiListBox   of writeRequirements
                                                              writeGlobalFunction endof
				comment;
                                         s"  \ Not executable: " ( default ) rot
                                endcase ?dup
                                if      append GetFunction: ThisControl 
					2dup ?BLonly 
					if  2drop GROUP?: ThisControl
						if   s" SetControl#" 
						else s" noop" 
                                        	then 
					then append GROUP?: ThisControl
					if   GetName: ThisControl GetObject# append
                                       	then s"  SetFunc: " append 
					GetName: ThisControl  append&crlf
                                        +crlf
                                else    drop
                                then
                        then
                loop    ;

: writeGlobalRequirements  ( -- )  \ write additional global Requirements
                          \ added DB 2009-0408 - to be integrated in writeFunctions !
                #controls 0= ?exit
                +crlf   \ force start on new line
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        Global?: ThisControl
                        if      GetType: ThisControl
                                case    TypeCheckBox       of writeRequirements endof
                                        TypeRadioButton    of writeRequirements endof
                                        TypeListBox        of writeRequirements endof
                                        TypeComboBox       of writeRequirements endof
                                        TypeHorizScroll    of writeRequirements endof
                                        TypeVertScroll     of writeRequirements endof
                                        TypeLabel          of writeRequirements endof
                                        TypeComboListBox   of writeRequirements endof
                                        TypeStaticBitmap   of writeRequirements endof
                                        TypeMultiListBox   of writeRequirements endof
                                        TypeTextBox        of writeRequirements endof
                                        TypeMultiLineBox   of writeRequirements endof
                                        TypeScrollTextBox  of writeRequirements endof
                                        TypeDrawBox        of writeRequirements endof
                                         s"  \ No additional requirements: " ( default ) rot
                                endcase drop
                        then
                loop    ;


: writecontrols { \ ctrl -- }  \ write local controls in form
                #controls 0= ?exit
                +crlf   \ force start on new line
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        Global?: ThisControl not
                        if      GetType: ThisControl
                                case
                                        TypeTextBox        of Spinner?: ThisControl
                                                              if   s" SpinnerControl "
                                                              else s" TextBox "
                                                              then                  endof
                                        TypeGroupBox       of s" GroupBox "         endof
                                        TypePushButton     of s" PushButton "       endof
                                        TypeCheckBox       of s" CheckBox "         endof
                                        TypeRadioButton    of Group?: ThisControl
                                                              if   s" GroupRadioButton "
                                                              else s" RadioButton "
                                                              then                  endof

                                        TypeBitmapButton   of s" BitmapButton "     endof
                                        TypeListBox        of s" ListBox "          endof
                                        TypeComboBox       of s" ComboBox "         endof
                                        TypeHorizScroll    of s" HorizScroll "      endof
                                        TypeVertScroll     of s" VertScroll "       endof
                                        TypeLabel          of s" Label "            endof
                                        TypeComboListBox   of s" ComboListBox "     endof
                                        TypeMultiLineBox   of s" MultiLineTextBox " endof
                                        TypeStaticBitmap   of s" StaticBitmap "     endof
                                        TypeMultiListBox   of s" MultiListBox "     endof
                                        TypeFileWindow     of s" FileWindow "       endof
                                        TypeTabControl     of s" TabControl "       endof
                                        TypeDrawBox        of s" DrawBox "          endof
                                        TypeScrollTextBox  of s" ScrollTextBox "    endof
                                        TypeGeneric        of write-child-params
                                                              s" "                  endof
                                        s"  \ UnKnown Control " ( default ) rot
                                endcase ?dup
                                if      append GetName: ThisControl append&crlf
                                 else    drop
                                then
                        then
                loop    ;

\ More Listings	DB 2009-0430-0502

: GetControlList { \ ctrl -- }  \ type controls to console
                #controls 0= ?exit
		+crlf
		s" List of " append frmName count append s" -Controls:" append&crlf
		+crlf #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        1 +tabs i (.) append 1 +tabs
			GetName: ThisControl append
                        Global?: ThisControl 0=
			     if	  s"  \ local Control" append 
			     then +crlf
                loop ;

:M GetControlList:  ( -- addr len )
                initbuffer
                GetControlList
                TheBuffer
;M

:M TypeControlList:  ( -- )
	ShowConsole
                initbuffer
                GetControlList
                TheBuffer lType
;M

variable FormFunction?
variable Function?

: GetFormFunctionsList ( -- )		
	FormFunction? off
	BitmapUsed 
	 if   +crlf s" \ Bitmaps:" append&crlf 
	      true to Bitmapstate WriteGlobalBitmapDefs false to Bitmapstate
         then
    	 FormNeedsFunction: ActiveForm count ?dup	\ added DB 2009-0527
	 if   +crlf s" \ Words to execute before Loading of Form:" append&crlf 
	      FormFunction? on append&crlf
	 else drop
	 then	
    	 FormInitFunction: ActiveForm count ?dup
	 if   +crlf s" \ Words to execute before Initialization of Form:" append&crlf 
	      FormFunction? on append&crlf
	 else drop
	 then	
    	FormInitializedFunction: ActiveForm count ?dup
	 if   +crlf s" \ Words to execute when Form is initialized:" append&crlf
	      FormFunction? on append&crlf
	 else drop
	 then
    	 FormPaintFunction: ActiveForm count ?dup         \ added DB 2012-0428
	 if   +crlf s" \ Words to execute on painting:" append&crlf
	      FormFunction? on append&crlf
	 else drop
	 then
    	 FormSizeFunction: ActiveForm count ?dup          \ added DB 2012-0428
	 if   +crlf s" \ Words to execute on sizing:" append&crlf
	      FormFunction? on append&crlf
	 else drop
	 then
    	 FormDoneFunction: ActiveForm count ?dup
	 if   +crlf s" \ Words to execute when done:" append&crlf
	      FormFunction? on append&crlf
	 else drop
	 then ;

variable Group-Function

: GetFunctionList { \ ctrl -- }  \ copy Functions to console	\ DB 2009 05 added
                #controls 0= ?exit
		Group-Function off +crlf
		s" \ List of " append frmName count append s" -Functions:" append&crlf
		GetFormFunctionsList
		#controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        GetRequirements: ThisControl ?dup
			if   2dup SearchTheBuffer 0=				    \ added DB 2009 0611
				if   +crlf +crlf  s" \ Requirements for " append GetName: ThisControl  append&crlf
				     +crlf  append&crlf Function? on
				else 2drop
				then
			else	drop
			then
			GetFunction: ThisControl ?dup
                        if	+crlf +crlf GetEdFunction: ThisControl dup
					if   s" : " 
					else s" \ "				    \ added DB 2099 0611
					then append >r >r
				 append s"      \ " append 
				GROUP?: ThisControl 				    \ added DB 2009 0611
					if   s" Group-" append Group-Function on
					then s" Function of " append
				GetName: ThisControl append
                        	Global?: ThisControl 0=
			     	if	s"  as local Control" append then 
                        	+crlf  r> r> ?dup 
				if 	 append s"  ;" append
				else	drop
				then	+crlf Function? on
			else 	drop GROUP?: ThisControl 				    \ added DB 2009 0611
					if   +crlf s" \ SetControl#" append GetName: ThisControl GetObject# append
					     s"      is Group-Function of " append GetName: ThisControl append&crlf
					     Group-Function on
					then
			then
                loop  
		Function? @ 0= 	if   FormFunction? @
					if   +crlf s" \ no other" append
				        else s"  no" append
					then s"   Function used" append
				else Group-Function @
					if   +crlf s" \ Group-Functions always set Control# !" append&crlf
					then +crlf s" \ End of Functionlist " append frmName count  append
				then +crlf +crlf ;

:M GetFunctionList:  ( -- addr len )		
                initbuffer
                GetFunctionList
                TheBuffer
;M

:M TypeFunctionList:  ( -- )
	ShowConsole
                initbuffer
                GetFunctionList
                TheBuffer lType
;M

\ Additions for Bitmap! DB 2009-0430

variable CaseWriting

: WriteCase ( -- )
	CaseWriting @ 0=
           if   +crlf   \ force start on new line
		s" : Bitmap! ( handle n -- )" append&crlf	\ changed DB 2009 0605
		1 +tabs s" case" append&crlf
		CaseWriting on
	   then ;

: WriteEndCase ( -- )		\ includes Alias 	\ DB 2009-0501
	CaseWriting @
	   if	1 +tabs s" drop endcase ;" append&crlf
		+crlf
		s" ' Bitmap! alias " append frmName count append s" -Bitmap!" append&crlf 
		CaseWriting off
	   then ;

: WriteBitmapSelect { \ ctrl -- }  \ write case of global bitmap controls
                #controls 0= ?exit
		CaseWriting off
		Decimal
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        Global?: ThisControl
			GetType: ThisControl TypeBitmapButton =
			GetType: ThisControl TypeStaticBitmap = or and
			   if	WriteCase 1 +tabs i (.) append 
				s"  of setimage: " append
				GetName: ThisControl append s"  endof" append&crlf
			   then
                loop WriteEndCase ;

\ End of Additions for Bitmap! 

: writeClassInit        ( -- )
                        +crlf
                        s" :M ClassInit:   ( -- )" append&crlf
                        2tabs
                        s" ClassInit: super" append&crlf
                        GetSuperClass: self CHILD-CLASS =
                        if      2tabs s" +dialoglist  \ allow handling of dialog messages" append&crlf
                                2tabs NextId #append s"  to id     \ set child id, changeable" append&crlf
                        then    2tabs
                        s" ;M" append&crlf ;

: ?write-child-style    ( -- )
                    GetSuperClass: self CHILD-CLASS <> ?exit
                    +crlf
                    s" :M Display:     ( -- ) \ unhide the child window" append&crlf
                    2tabs s" SW_SHOWNORMAL Show: self ;M" append&crlf
                    +crlf
                    s" :M Hide:        ( -- )   \ hide the...aughhh but you know that!" append&crlf
                    2tabs s" SW_HIDE Show: self ;M" append&crlf
                    ChildState: self 0= ?exit   \ not initially invisible
                    +crlf
                    s" :M WindowStyle:  ( -- style )" append&crlf
                    2tabs s" WS_CHILD " append&crlf
                    2tabs s" ;M" append&crlf
                    +crlf
                    s" :M Start:       ( Parent -- )" append&crlf
                    2tabs s" to parent" append&crlf
                    2tabs s" register-child-window drop" append&crlf
                    2tabs s" create-child-window to hWnd ;M " append&crlf ;

: writeWindowStyle      ( -- )
                        ?write-child-style
                        GetSuperClass: self DIALOG-CLASS <> \ ?exit   \ don't write style for child windows or mdi dialogs
                        if     GetSuperClass: self SUPER-CLASS =      \ added DB 2012-0407
                                 if   +crlf
                                      s" :M WindowStyle:  ( -- style )" append&crlf
                                      2tabs s" WindowStyle: SUPER " append&crlf
                                      2tabs s" ;M" append&crlf 
                                 then     
                        else +crlf
                             s" :M WindowStyle:  ( -- style )" append&crlf
                             2tabs s" WS_POPUPWINDOW WS_DLGFRAME or " append&crlf
                             2tabs s" ;M" append&crlf 
                        then ;

: writeWindowTitle      ( - )
                        +crlf
                        s" :M WindowTitle: ( -- ztitle )" append&crlf
                        2tabs
                        [char] z cappend [char] " cappend 1 +spaces frmTitle count append 
                        TimeStamp @ if 5 +spaces GET-LOCAL-TIME TIME-BUF dup >DATE" append 2 +spaces >TIME" append then
                        [char] " cappend +crlf
                        2tabs
                        s" ;M" append&crlf ;

: writeStartSize        ( -- )
                        +crlf
                        s" :M StartSize:   ( -- width height )" append&crlf
                        2tabs
                        frmWidth #append zooming @
				if s"  zoom" append frmName count append s"  " append then 	\ DB 090228 form zoom
			frmHeight #append zooming @
				if s"  zoom" append frmName count append then		\ DB 090228 form zoom
			+crlf
                        2tabs s" ;M" append&crlf ;

: writeStartPos         ( -- )
                        GetSuperClass: self DIALOG-CLASS <> \   ?exit
                        GetSuperClass: self SUPER-CLASS <> and ?exit      \ added DB 2012-0407
                        +crlf
                        s" :M StartPos:    ( -- x y )" append&crlf
                        2tabs
                        SaveScreen?
                        if  s" XYPos"
                        else    frmXPos #append 1 +spaces frmYPos (.)
                        then    append&crlf
                        2tabs s" ;M" append&crlf ;

: writeWM_COMMAND       ( -- )
                        +crlf GetSuperClass: self MDIDIALOG-CLASS =
                        if    s" :M ON_COMMAND:  ( h m w l -- res )" append&crlf
                        else  s" :M WM_COMMAND   ( h m w l -- res )" append&crlf
                        then  with-menubar                                \ added DB 2012-0414
                        if   2tabs s" dup 0=      \ id is from a menu if lparam is zero" append&crlf
                             2tabs s" if        over LOWORD CurrentMenu if dup DoMenu: CurrentMenu then" append&crlf
                             2tabs s"           CurrentPopup if dup DoMenu: CurrentPopup then drop" append&crlf
                             2tabs s" else      " append true
                        else 2tabs false
                        then  
                        s" over LOWORD ( ID ) self   \ object address on stack" append&crlf
                        2tabs s" WMCommand-Func ?dup    \ must not be zero" append&crlf
                        2tabs s" if  " append 1 +tabs s" execute" append&crlf
                        2tabs s" else" append 1 +tabs s" 2drop   \ drop ID and object address" append&crlf
                        if    2tabs s" then" append&crlf
                        then  2tabs s" then" append 1 +tabs s" 0 ;M" append&crlf
                        \ write function to set OnWMCommand
                        +crlf s" :M SetCommand:  ( cfa -- )  \ set WMCommand function" append&crlf
                        2tabs s" to WMCommand-Func" append&crlf
                        2tabs s" ;M" append&crlf ;

: writeOnPaint          ( -- )
                        +crlf
                        s" :M On_Paint:    ( -- )" append&crlf
                        2tabs s" 0 0 GetSize: self Addr: FrmColor FillArea: dc" append&crlf
                        2tabs s" \ FormPaint-Function:" append&crlf       \ added DB 2012-0428
                        2tabs FormPaintFunction: self count append&crlf
                        2tabs s" ;M" append&crlf ;

: writeOnSize           ( -- )
                        with-statusbar not ?exit
                        +crlf
                        s" :M On_Size:      ( -- )" append&crlf
                        2tabs s" Redraw: TheStatusBar" append&crlf
                        2tabs s" \ FormSize-Function:" append&crlf       \ added DB 2012-0428
                        2tabs FormSizeFunction: self count append&crlf
                        2tabs s" ;M" append&crlf ;

: WriteInitialized            ( -- )
                        +crlf
                  \      s" :M Close:" append 1 +tabs s" ( -- )" append&crlf  \ removed because function changed to FormInitialized-Functions: \ DB 2012 0116
                        \ we close any FileWindow object, avoid head scratching bugs :-p
                        TypeFileWindow  get-array       \ do we have any?
                        if      #controls 1+ 1
                                do      i >Link#: ControlList
                                        Data@: ControlList to ThisControl
                                        GetType: ThisControl TypeFileWindow =
                                        if       2tabs s" Close: " append
                                                 GetName: ThisControl append&crlf
                                        then
                                loop
                        then    2tabs s" \ FormInitialized-Functions:" append&crlf
			2tabs FormInitializedFunction: self count append&crlf
                  \      2tabs s" Close: super" append&crlf \ removed because function changed to FormInitialized-Functions: \ DB 2012 0116
                        2tabs s" ;M" append&crlf ;

: WriteDeleteBitmaps { \ ctrl -- }  \ write delete of bitmaps				\ DB 2009 0605
                #controls 0= ?exit
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
			GetType: ThisControl TypeBitmapButton =
			GetType: ThisControl TypeStaticBitmap = or 
			   if	GetBitmap: ThisControl 
				(WriteDeleteBitmap)
			   then
                loop
;

: writeOnDone   ( -- )
                +crlf
                s" :M On_Done:    ( -- )" append&crlf
                2tabs s" Delete: WinFont" append&crlf
                write-delete-fonts        	\ delete any other fonts created
		WriteDeleteBitmaps  		\ delete loaded Bitmaps \ added DB 2009 0605
		3 to BitmapState		\ added DB 2009 0707
		WriteFormCases +crlf		\ added DB 2009 0702
                 GetSuperClass: self DIALOG-CLASS =
                if  SaveScreen?
                    if      2tabs s" originx originy 2to XYPos" append&crlf
                    then
                    \ write following code only if this is a modal form
                    modal
                    if      2tabs s" ParentWindow: self   \ if modal form re-enable parent" append&crlf
                            2tabs s" if   1 ParentWindow: self Call EnableWindow drop" append&crlf
                            2tabs s"      \ reset focus to parent if we have one"
                            append&crlf
                            2tabs s"      ParentWindow: self Call SetFocus drop"
                            append&crlf
                            2tabs s" then" append&crlf
                    then
                then    2tabs s" \ FormDone-Function:" append&crlf
			2tabs FormDoneFunction: self count append&crlf
		turnkey?
                if	2tabs s" 0 call PostQuitMessage drop     \ terminate application" append&crlf \ DB 2009-0314
                then	2tabs s" On_Done: super" append&crlf
                2tabs s" ;M" append&crlf ;

: writeParentWindow     ( -- )
        GetSuperClass: self DIALOG-CLASS <> ?exit
                +crlf
                s" \ if this form is a modal form a non-zero parent must be set" append&crlf
                s" :M ParentWindow:  ( -- hwndparent | 0 if no parent )" append&crlf
                2tabs s" hWndParent" append&crlf
                2tabs s" ;M" append&crlf
                \ write function for setting parent window
                +crlf s" :M SetParentWindow:  ( hwndparent -- ) \ set owner window" append&crlf
                2tabs s" to hWndParent" append&crlf
                2tabs s" ;M" append&crlf ;

: writeclosing  ( -- )
                +crlf
                s" ;Object" append&crlf ;

: appendAddAppIcon ( -- )			\ DB 2009-0307
		s"append s" src\res\visualFORTH.ico" append "append bl cappend
		s"append frmName count append s" .exe" append "append 
                s"  Prepend<home>\ AddAppIcon" append&crlf
;

: frmName>ActiveFormName ( -- )			\ added for Turnkey DB 2009-0310
		frmName count dup >r ActiveFormName 1+ swap cmove
		r> ActiveFormName c!
;

: writestartandstop  ( -- )				\ removed from writeturnkey DB 2009-0601
                +crlf
		s" : Start  ( -- )" append&crlf
		\ things to do to start the application
		2tabs s" start: " append frmName count append&crlf
		2tabs s"  ;" append&crlf 
                +crlf
		s" ' Start alias " append frmName count append s" -Start" append&crlf 
                +crlf
		s" : Stop  ( -- )" append&crlf
		\ things to do to close the application
		2tabs s" close: " append frmName count append&crlf
		2tabs s"  ;" append&crlf 
                +crlf
		s" ' Stop alias " append frmName count append s" -Stop" append&crlf 
                +crlf ;

: writezooming  ( -- )				\ removed from writeturnkey DB 2009-0601
	zooming @
	if	s" : %Zoom  ( -- )" append&crlf
		\ things to do to zoom the application
		2tabs s" 25 max to zoom% stop start" append&crlf	\ >25% DB 2009-0404
		2tabs s"  ;" append&crlf 
                +crlf
		s" ' %Zoom alias " append frmName count append s" %Zoom" append&crlf 
		+crlf
	then	;

: writeturnkey  ( -- )				\ DB 2009-0307
	turnkey? if	\ the remaining part only to be loaded with turnkey, otherwise Test will abort vF! DB 2009-0426
		Standalone @						\ DB 2009-0405
			if s" needs NoConsole.f" append&crlf +crlf then
		s" \- MessageLoop : MessageLoop { | pMsg -- } 7 cells LocalAlloc: pMsg Begin 0 0 0 pMsg call GetMessage While pMsg handlemessages drop Repeat ;" append&crlf
                +crlf
                s" needs Resources" append&crlf +crlf
		s" : Main  ( -- )" append&crlf
		\ things to do to start the application
		2tabs s" start" append&crlf 
\		2tabs s" ApplicationBar SetMenuBar: " append frmName count append&crlf 
		\ enter the message loop
                Interpreter?: self                              \ DB 2012-0511
                  if   2tabs s" start MessageLoop  ;"
                  else s" Turnkeyed?" append&crlf
		       2tabs s" IF   MessageLoop bye" append&crlf
		       2tabs s" THEN ;"
                  then append&crlf +crlf +crlf
		Standalone @					\ DB 2009-0404
				\ Setup the console I/O
			if   2tabs s" NoConsoleIO" append&crlf
				\ Tell Imageman that we don't need the w32fconsole.dll.
			     2tabs s" NoConsoleInImage" append&crlf
			then	\ Create the exe-file
        	2tabs s" \ Create the exe-file" append&crlf
        	2tabs s" &forthdir count &appdir place" append&crlf
                Interpreter?: self                              \ DB 2012-0511
                  if   s"  CONSOLEHIDDENBOOT ' Main save "
		  else 2tabs s" ' Main turnkey " 
                  then append frmName count append s" .exe" append&crlf 
		\ add the Application icon to the EXE file
		2tabs appendAddAppIcon
		2tabs s" 1 pause-seconds" append&crlf
		2tabs s" bye" append&crlf
		then frmName>ActiveFormName ;


: writeWMNotify ( -- )
                TypeTabControl get-array 0= ?exit   \ we don't have a tabcontrol
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        GetType: ThisControl TypeTabControl = ?leave
                loop    +crlf
                s" :M WM_NOTIFY  ( h m w l -- f )" append&crlf
                s" \ if this form has more than one tab control this handler will need to be modified"
                append&crlf
                2tabs s" Handle_Notify: " append GetName: ThisControl append&crlf
                2tabs s" ;M" append&crlf ;

: startpushbutton  ( -- )
                writecommoncode
                writefont
                \ write style only if not default
                Orientation: ThisControl dup 1 <>
                if      case
                                0       of      s" BS_LEFT"         endof
                                2       of      s" BS_RIGHT"        endof
                                s" BS_LEFT" rot
                        endcase writestyle
                else    drop
                then    GetToolTip: ThisControl ?dup		\ added DB 2009-0410
                if      +crlf 2tabs s"append append ( tooltip ) "append 
                        s"  ToolString: " append GetName: ThisControl append \ &crlf
                else    drop
                then	writetext +crlf ;

: starttextbox  ( -- )
                \ write style only if not default
                GetType: ThisControl TypeMultiLineBox =
                GetType: ThisControl TypeScrollTextBox = or
                Orientation: ThisControl 0<> and
                if      Orientation: ThisControl
                        case
                                1       of      s" ES_CENTER"       endof
                                2       of      s" ES_RIGHT"        endof
                                s" ES_LEFT" rot
                        endcase +crlf 2tabs append
                        s"  AddStyle: " append GetName: ThisControl append
                then    writecommoncode writefont +crlf
                GetType: ThisControl TypeTextBox = Spinner?: ThisControl and
                if       2tabs s" \ set default values, easily changed " append&crlf
                         2tabs GetSpinnerMinMax: ThisControl swap (.) append bl cappend (.) append bl cappend \ added DB 2009-1122
                               s" SetRange: " append GetName: ThisControl append&crlf
                         2tabs GetTitle: ThisControl 2dup (number?) 	\ added DB 2009-1120
                               if 2drop else 4drop s" 0" then append	\ added DB 2009-1120 
                         s"  SetValue: " append GetName: ThisControl append&crlf
                then	 
                comment: \ removed "Toolstring not understood by class textbox" DB 2012-0408
                         GetToolTip: ThisControl ?dup		\ added DB 2009-0412
                if       2tabs s"append append ( tooltip ) "append 
                         s"  ToolString: " append GetName: ThisControl append&crlf
                else     drop
                then	
                comment;
                 ;

: startlabel    ( -- )
                writecommoncode
                writefont
                \ write style only if not default
                Orientation: ThisControl ?dup
                if      case
                                1       of      s" SS_CENTER"       endof
                                2       of      s" SS_RIGHT"        endof
                                s" SS_LEFT" rot
                        endcase writestyle
                then    writetext +crlf ;

: startlistbox  ( -- )
                writecommoncode
		GetToolTip: ThisControl ?dup		\ added DB 2009-0412
                if      +crlf 2tabs s"append append ( tooltip ) "append 
                        s"  ToolString: " append GetName: ThisControl append \ &crlf
                else    drop
                then	writefont +crlf ;

: startcombobox ( -- )
                writecommoncode
                GetToolTip: ThisControl ?dup		\ added DB 2009-0412
                if      +crlf 2tabs s"append append ( tooltip ) "append 
                        s"  ToolString: " append GetName: ThisControl append \ &crlf
                else    drop
                then	writefont +crlf ;

: startradiobutton  ( -- )
                writecommoncode
                writefont
                \ write style only if not default
                Orientation: ThisControl ?dup
                if      case
                                0       of      s" BS_LEFTTEXT"     endof	\ changed to justification "left"! DB 2009-1122
                                1       of      s" BS_CENTER"       endof
                                2       of      s" BS_RIGHT"        endof
                    \            3       of      s" BS_LEFTTEXT"     endof	\ changed to justification "left"! DB 2009-1122
                        endcase writestyle
                then    GetToolTip: ThisControl ?dup		\ added DB 2009-0412
                if      +crlf 2tabs s"append append ( tooltip ) "append 
                        s"  ToolString: " append GetName: ThisControl append \ &crlf
                else    drop
                then	writetext +crlf ;

: startcheckbox ( -- )
                writecommoncode
                writefont
                \ write style only if not default
                Orientation: ThisControl ?dup
                if      case			\ corrected with swap DB 2010-0429	
                                0       of      s" BS_LEFTTEXT"     endof	
                                1       of      s" BS_CENTER"       endof
                                2       of      s" BS_RIGHT"        endof
                        endcase writestyle
                then    GetToolTip: ThisControl ?dup		\ added DB 2009-0412
                if      +crlf 2tabs s"append append ( tooltip ) "append 
                        s"  ToolString: " append GetName: ThisControl append \ &crlf
                else    drop
                then	writetext +crlf ;

: startgroupbox ( -- )
                writecommoncode
                writefont
                Orientation: ThisControl ?dup
                if      case
                                1       of      s" BS_CENTER"       endof
                                2       of      s" BS_RIGHT"        endof
                                s" BS_LEFT" rot
                        endcase writestyle
                then    writetext +crlf ;

: (Load:Bitmaps) { \ ctrl -- }  \ load bitmaps				\ DB 2009 0607
                #controls 0= ?exit
                #controls 1+ 1
                do      i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        Global?: ThisControl
			GetType: ThisControl TypeBitmapButton =
			GetType: ThisControl TypeStaticBitmap = or and
			   if	 GetBitmap: ThisControl 
				 (Load:Bitmap)
			   then
                loop 
;

: Load:Bitmaps  ( -- )							\ DB 2009 0707
		(Load:Bitmaps) 
		2 to BitmapState	
		WriteFormCases	
;

: startbitmapbutton   ( -- )
                writecommoncode +crlf
                2tabs GetBitmap: ThisControl dup			\ Changed DB 2009 0610
			if   4 - ( bitmap ) 
			else 2drop s" vF"
			then append s"  SetImage: " append GetName: ThisControl append&crlf    
                GetToolTip: ThisControl ?dup
                if      2tabs s"append append ( tooltip ) "append
                        s"  ToolString: " append GetName: ThisControl append \ &crlf
                else    drop
                then	+crlf ;


: startstaticbitmap     ( -- ) \
\ for these controls image must be set before sizing control so image size
\ will be adjusted correctly
                +crlf
                2tabs s" self Start: " append GetName: ThisControl append&crlf
                2tabs s" SS_REALSIZECONTROL +style: " append GetName: ThisControl append&crlf
                2tabs GetBitmap: ThisControl dup			\ added for groups
			if   4 - ( bitmap ) 
			else 2drop s" vF"
			then append s"  SetImage: " append GetName: ThisControl append&crlf    
                2tabs Origin: ThisControl swap
\ Zooming								\ DB 090426
		#append zooming @
		if s"  zoom" append frmName count append s"  " append then 
		#append zooming @
		if s"  zoom" append frmName count append s"  " append then
                Dimensions: ThisControl swap 
		#append zooming @ 
		if s"  zoom" append frmName count append s"  " append then
		#append zooming @
		if s"  zoom" append frmName count append s"  " append then
                s" Move: " append GetName: ThisControl append
                Group?: ThisControl
                if      +crlf 2tabs s" WS_GROUP +Style: " append
                        GetName: ThisControl append
                then    +crlf ;

: starthorizscroll  ( -- )
                writecommoncode +crlf ;

: startvertscroll  ( -- )
                writecommoncode +crlf ;

: startfilewindow  ( -- )
                writecommoncode +crlf ;

: starttabcontrol ( -- )
                writecommoncode
                writefont
                +crlf ;

: startdrawbox ( -- )
                writecommoncode
                writefont
                +crlf ;

: writeOnInit   ( -- )  \ create control objects in form
                +crlf 102 to BMP-ID		\ Set start for BMP-IDs	\ DB 2009 0609
                s" :M On_Init:     ( -- )" append&crlf
                2tabs s"append s" MS Sans Serif" append "append
                s"  SetFaceName: WinFont" append&crlf
                2tabs s" 8 Width: WinFont" append&crlf
( create font ) 2tabs s" Create: WinFont " append&crlf
        +crlf
        2tabs s" \ set form color to system color" append&crlf
        2tabs s" COLOR_BTNFACE Call GetSysColor NewColor: FrmColor" append&crlf  +crlf
        with-statusbar
        if      2tabs s" self Start: TheStatusBar" append&crlf
                with-multistatusbar @
                if   2tabs frmName count append s" .bardivisions " append
                     frmName count append s" .bardivisions# SetParts: TheStatusBar" append&crlf
                then +crlf
        then
        with-menubar                             \ DB 2012-0414
        if      2tabs FormMenuBar: self count 4 - append
                      s"  SetMenuBar: self" append&crlf
                +crlf
        then    2tabs s" \ FormInit-Functions:" append&crlf		\ This text has to be the same in Load:Bitmaps !
		Load:Bitmaps 						\ DB 090607
		2tabs FormInitFunction: self count append&crlf
                #controls 1+ 1
                ?do     i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        GetType: ThisControl
                        case
                                TypeTextBox        of starttextBox       endof
                                TypeGroupBox       of startGroupBox      endof
                                TypePushButton     of startPushButton    endof
                                TypeCheckBox       of startCheckBox      endof
                                TypeRadioButton    of startRadioButton   endof
                                TypeBitmapButton   of startBitmapButton  endof
                                TypeListBox        of startListBox       endof
                                TypeComboBox       of startComboBox      endof
                                TypeHorizScroll    of startHorizScroll   endof
                                TypeVertScroll     of startVertScroll    endof
                                TypeLabel          of startLabel         endof
                                TypeStaticBitmap   of StartStaticBitmap  endof
                                TypeComboListBox   of startComboBox      endof
                                TypeMultiLineBox   of startTextBox       endof
                                TypeScrollTextBox  of startTextBox       endof
                                TypeMultiListBox   of startListBox       endof
                                TypeFileWindow     of startfilewindow    endof
                                TypeTabControl     of starttabcontrol    endof
                                TypeDrawBox        of startdrawbox       endof
                        endcase
                loop    GetSuperClass: self DIALOG-CLASS =
                if  modal
                    if      +crlf 2tabs s" ParentWindow: self   \ if this is a modal form disable parent" append&crlf
                            2tabs s" if   0 ParentWindow: self Call EnableWindow drop then" append
                    then
                then        LocalFunction? @ if writeDefaultCommandLink then
                            +crlf \ 2tabs s" ;M" append&crlf  \ removed to add FormInitialized-Functions: \ DB 2012 0116
;

:M Validate:    ( -- )  \ check form and controls for spaces in names
\ in case they were loaded with such
               frmname count -trailing bl scan nip
               s" Spaces not allowed in form name!" ?Errorbox
               #controls 0= ?exitm
               #controls 1+ 1
               do       i >Link#: ControlList
                        Data@: ControlList to ThisControl
                        GetName: ThisControl -trailing bl scan nip
                        if       join$(  s" ( "
					 GetName: ThisControl
					 s"  ) spaces not allowed in control name!"
                                 )join$ true swap count ?Errorbox
                        then
               loop    ;M

: WritetoBuffer ( -- )
		zoomdefDone off    \ to write Zoom-Definitions 2009-02-28 DB
		zoomfont off 	   \ to mark writing of Zoom-Definitions 2009-11-20 DB
                LocalFunction? off \ switch for LocalFunction handling
                Validate: self
                writeneeds
                writeglobals
		WriteBitmapSelect		\ Additions for Bitmap! DB 2009-0430
		writeThisFormNeeds		\ positioned here to make Bitmap! usable DB 2009 0622
                writeopening
                writecontrols
                writeLocalFunctions
                LocalFunction? @ if writeDefaultCommand then
                writeClassInit
                writeWindowStyle
                writeParentWindow
                writeWindowTitle
                writeStartSize
                writeStartPos
                writeWMNotify
                writeWM_COMMAND
                writeOnInit
                WriteInitialized
                writeOnPaint
                writeOnSize
                WriteOnDone
                writeClosing 
		writestartandstop	\ got here from writeturnkey DB 2009 0601
		writezooming		\ got here from writeturnkey DB 2009 0601
		writeGlobalRequirements \ added to have more space for requirements DB 200-0408
                writeFunctions		\ to type in a word for execution  2008-07-12 DB
\		writeMenuBar		\ preparing TurnKey but not used DB 2009-0307
		writeturnkey		\ preparing TurnKey DB 2009-0307
                ;

:M GetBuffer:   ( -- addr len )
                initbuffer
                WriteToBuffer
                TheBuffer
                ;M

:M UninitedBuffer: ( -- addr len )
                   WriteToBuffer TheBuffer ;M

:M TestChildDialog: ( -- addr len )
        initbuffer
        s" :Object TestChildForm    <Super Window" append&crlf
        +crlf
        s" :M On_Init:  ( -- )" append&crlf
        2tabs s" self Start: " append FormName: self count 2dup 2>r append&crlf
        2tabs s" 0 0 StartSize: " append 2r@ append s"  Move: " append 2r@ append&crlf
        2tabs s" ;M" append&crlf
        +crlf
        s" :M WindowTitle: ( -- zstring )" append&crlf
        2tabs 'z' cappend "append s"  Test Window for " append 2r@ append "append
        s"  ;M" append&crlf
        +crlf
        s" :M StartSize:  ( -- )" append&crlf
        2tabs s" StartSize: " append 2r> append s"  ;M" append&crlf
        +crlf
        s" ;Object" append&crlf
        s" Start: TestChildForm" append&crlf ;M

:M TestMDIDialog: ( -- addr len )
        initbuffer
        s" MENUBAR MDIMenu" append&crlf
            s" POPUP " append "append s" &File" append "append +crlf
        s" MENUITEM " append "append s" E&xit  \tAlt-F4" append "append  s"  ;" append&crlf
            s" POPUP " append "append s" &Window" append "append +crlf
        s" ENDBAR" append&crlf
        s" :Object Frame        <Super MDIFrameWindow" append&crlf
        +crlf
                s" :M ClassInit:   ( -- )" append&crlf
                s" ClassInit: super" append&crlf
                s" MDIMenu to CurrentMenu" append&crlf
                s" ;M" append&crlf
        s" :M On_Size:  ( -- )" append&crlf
        2tabs s" 0 0 Width Height Move: MDIClient ;M" append&crlf
        +crlf
        s" ;Object" append&crlf
        s" Start: Frame" append&crlf
        s" MDICLientWindow: Frame Start: " append FormName: self count append&crlf ;M

: textfile      ( -- addr cnt ) \ return text (.frm) file name
                new$ >r formname count 
		"minus-ext" r@  place
                s" .frm" r@ +place r> count ;

:M TextFile:    ( addr cnt -- )
                textfile
                ;M

: OverWrite?$ ( -- addr )		\ Form Name added	DB 2009-0528 
                FormTitle: ActiveForm count new$ dup>r place
                s" -form file already exist, overwrite?" r@ +place
                r> count asciiz ;

: OverWrite?    ( \ temp$ -- f )
		OverWrite?$		\ Form Name added	DB 2009-0528 
                z" Warning!" MB_YESNO MB_ICONQUESTION or
                MessageBox: TheMainWindow IDYES = ;

: WriteForm     { flag \ file$ -- } \ flag is true if showing messages
                max-path LocalAlloc: file$
                IsNewForm?
                if      SaveIt?
                        if      SaveForm 		  \ save it first
                        else    exit
                        then
                then    textfile file$ place 		  \ add new extension
                file$ count file-status nip 0=		  \ already exist?
                if      SetFocus: self OverWrite? 0= ?exit
                then    GetBuffer: self nip 		  \ do we have info?
                if      file$ count SetName: FormFile
                        Create: FormFile ?exit
                        TheBuffer Write: FormFile
                        Close: FormFile			  \ close first then
                        ?exit				  \ exit on error
                        flag 0= ?exit
                        join$(  s" File "
				file$ count
				s"  has been successfully compiled!"
                        )join$ 1+
                        z" Yes!"			  \ title
                        MB_OK 				  \ style
                        MessageBox: TheMainWindow
                else    flag s" Nothing compiled!" ?MessageBox
                then    ;

:M CompileForm: ( -- )
                true WriteForm
                ;M

:M Compile:     ( -- )
                false WriteForm ;M

\ Addtions for Turnkey						\ DB 2009-0507

: Name.ftk>CompiledFormName ( addr cnt -- )			\ added for Turnkey DB 2009-0311
	28 min dup >r CompiledFormName 1+ swap cmove
	r> dup 4 + CompiledFormName dup >r c! r> +
	ascii . over 1+ c! ascii f over 2 + c! ascii t over 3 + c! ascii k swap 4 + c!
;

: MakeTurnkeyExe ( -- )				\ for turnkey purposes DB 2009-0310
	initbuffer
	s" cd .. & win32for Chdir Projects " append
	s" fload " append CompiledFormName count append&crlf
	s" TurnkeyLoader" BatchFileExecute
;

: WriteTurnkeyFile  ( -- f )	\ flag is true if okay!		\ DB 2009-0506
   ActiveForm 0= ?exit
   true to turnkey?
   GetBuffer: ActiveForm
   CompiledFormName count ?dup
   if	TheBuffer>File true
   else drop false
   then false to turnkey?
;

: (Turnkey)  ( -- ) \ make .exe-file from image in memory	\ for turnkey purposes DB 2009-0307
	ActiveForm 0= ?exit
	frmName count Name.ftk>CompiledFormName
\	true s" Making EXE-File started " ?MessageBox  
	WriteTurnkeyFile
	if	MakeTurnkeyExe
	\	CopyEXEtoForthDir
	else 	true s" Could not make EXE-file" ?MessageBox 
	then ;


:M Turnkey:	( -- )
	InitTimer	\ init the timer for this window
	(Turnkey)
;M

\ End of Turnkey-Additions

;Class

: NewForm (  -- addr )
   New> Form dup to ThisForm ;

: (AddNewForm)  (  --  )
   		DisableArray		\ added for convenience DB 2009-0430
                FormList 0=             \ if we don't have one
                if      New> Linked-List to FormList    \ create it
                then    Data@: FormList         \ an entry of zero means it's empty
                if      AddLink: FormList
                then    NewForm Data!: FormList
                1 +to formcount ;

: AddNewForm    ( -- )
                ['] (AddNewForm) catch ExitOnError ;

: .forms        ( -- )
                #forms 0= ?exit
                Link#: FormList >r
                cr ." FormList= " FormList u.
                #forms 1+ 1
                do      i >Link#: FormList
                        cr i . 8 spaces Data@: FormList dup u.
                        8 spaces
                        ?dup if FormName: [ ] count type then
                        cr
                loop    ." ActiveForm = " ActiveForm u. cr
                r> >Link#: FormList ;

: DisposeForms   ( -- )
                #Forms 0= ?exit         \ no forms open
                >FirstLink: FormList            \ start from the beginning
                #Forms 0                                        \ for each open form
                ?do     Data@: FormList dup to ThisForm
                        if      GetHandle: ThisForm             \ is it open?
                                if      SetFocus: ThisForm
                                        Close: ThisForm
                                then    ThisForm dispose        \ release memory
                                0 Data!: FormList               \ mark as empty
                                DeleteLink: FormList    \ and delete
                        then    >NextLink: FormList             \ next form
                loop    FormList Dispose 0 to FormList ;


\s
