\ source: FileEmit.f by BERTRAND Andr Jacques
\ **************************************************************************
\ Switches console screen outputs (emit, cr and type etc.) towards a textbox
\ **************************************************************************
\ file changed to textbox DB 2009-0319

\ Redirection Words: IsOutput, IsInput

anew -textboxemit.f

130 constant c/lmax
128 constant Rowmax
c/lmax Rowmax * constant OutputBufferMax

  64 value c/l		\ redefined, original definition \src\lib\block.f as constant
  16 value l/p	\ lines per page
   0 value RowStart
   0 value RowEnd
   0 value RowX
   0 value ColoumnY
c/l l/p * OutputBufferMax min value OutputSize
   0 value Output  		\ destination of output
\   0 value (Form-Output)	\ Key-Output of Form, belongs to Formkey.f \ deferred in fkernel.f
   0 value Form-Output	\ Key-Output of Form, belongs to Formkey.f \ was deferred in fkernel.f DB 2016-0221
                        \ changes made for Win32Forth 6.15.04 DB 2016-0221

: Spaces 	( n -- )		\ to circumvent ReduceBlanks	\ DB 2009-0405
	?dup if 0 do 0 ms bl emit loop then 
;


variable OutputBuffer OutputBufferMax allot
variable outputbufferpointer

: InitOutputBuffer ( -- )
	0 to RowX  0 to ColoumnY 0 to RowStart
 	OutputBuffer OutputBufferMax erase
	OutputBufferPointer off
;




: XY-BufferOffset ( -- n )
 	RowStart ColoumnY + 0 max
;


: OutputBufferfull? ( cnt -- flg )
	dup OutputBufferPointer @ + 
	swap XY-BufferOffset + max OutputBufferMax >
;

: TheOutputBuffer ( -- addr len )
	OutputBuffer OutputBufferPointer @
;

: OutputBufferAddress ( -- address )
	TheOutputBuffer +
;

internal

: GetNewRowStart 	( -- n )
	0 OutputSize 0 do OutputBuffer i + c@ 0= if drop i 1- leave then loop
;

: GetSecondLine		( -- n )
	0 OutputSize 0 do OutputBuffer i + c@ 10 = if drop i 1+ leave then loop
;

: MoveLineUp		( -- )
	OutputBuffer dup GetSecondLine dup >r +
	OutputBufferPointer @ r> - 1+  OutputSize min 
	dup OutputBufferPointer !
        0 do 2dup i + c@ swap i + c! loop 2drop
	OutputBuffer OutputBufferPointer @ + 1+ off
;

: XYappend        { addr cnt -- }
                cnt 0= ?exit
		cnt c/l min to cnt					\ added DB 2009-0402
                cnt OutputBufferfull? abort" Output-Buffer full!"
                addr OutputBuffer XY-BufferOffset + cnt 2dup + >r Cmove r> off
		ColoumnY cnt + to ColoumnY
		XY-BufferOffset cnt + OutputBufferPointer @ - dup
		0 > if   OutputBufferPointer +! 
		    else drop
		    then ;

\ internal

variable oEmit
variable oType
variable oCr
variable oGOTOXY
variable oCLS
variable oC/L
variable oL/P

: ClearRedirection ( -- )
		false to Output
		oEmit OFF oType OFF oCr OFF oGOTOXY OFF oCLS off
		oC/L off oL/P off
;

ClearRedirection \ clear it now...

: Redirection?	( -- f )
		oEmit @ 0<> ;

: SaveRedirection ( -- )
		Redirection? not \ only once
		if   defer@ emit   oEmit   !
		     defer@ type   oType   !
		     defer@ cr     oCr     !
		     defer@ GOTOXY oGOTOXY !
		     defer@ CLS    oCLS    !
		then ;

: SetRedirection ( emit_XT type_XT cr_XT GOTOXY_XT CLS_XT )
		is CLS is GOTOXY is cr is type is emit ;

: SetColRow ( c l -- )
  cr . . ." setcolrow unavailabler at W32F 6.14 "
;

: RestoreRedirection ( -- )
		Redirection? 
		if   oEmit @  oType @ oCr @ oGOTOXY @ oCLS @ SetRedirection
		     oC/L @ oL/P @ SetColRow 0 to Output
		     ClearRedirection
		then ;

external

: ReduceBlanks	( adr len -- adr len )	\ leaves one BL at the end 
	dup >r -trailing dup r> < 
		if    1+  
		then 
;


\ derived from \src\console\BasicWin.f SetText: and GetText:

: SetOutputText ( -- )   \ send Null-terminated text of OutputBuffer to window/control/output
	Output 0 <>
      		if   OutputBuffer
		     GetHandle: Output Call SetWindowText  drop \ ?WinError 
		then
;

: GetOutputText ( -- a n)   \ get window/control text
	Output 0 <>
	      	if   OutputSize Here 
		     GetHandle: Output Call GetWindowText \ DUP ?WinError
		     Here SWAP 1+	\ including zero at the end
		else 0 0
		then
;

: GetTypedInput ( -- a n )
	GetOutputText >r GetNewRowStart dup >r +
	r> r> swap -
;

: (type>TextBox)	( adr len -- )
	dup if	 ReduceBlanks
		 XYappend SetOutputText
	    else 2drop
	    then \ SetFocus: Fenster	\ cursor points on Fenster-Start DB 2009-0402
;

variable carfemit

: (carfemit)	( char -- addr 1 )
	carfemit c! carfemit 1
;

: emit>TextBox	( char -- )
	(carfemit) XYappend SetOutputText 
;

: Textbox>OutputBuffer ( -- )			\ has to be tested	DB 2009-0402
	Output if GetText: Output	?dup	\ gettext: only 260 bytes!
			if   OutputSize min 
			     dup OutputBufferPointer @ - ColoumnY + to ColoumnY
			     dup OutputBufferPointer !
			     OutputBuffer swap cmove
			else drop
			then
		then
;


variable KeyButtonQueue Maxstring allot	\ first character is pointer

: ClearKeyButtonQueue 	( -- )
	KeyButtonQueue Maxstring erase
;

: Type>Form-Output	( a n -- )
	KeyButtonQueue swap cmove
;

: Type2>Form-Output	( a1 n1 a2 n2 -- )	\ types 2 strings to Form-Output
	KeyButtonQueue 2dup + >r swap cmove r> 1- swap cmove
;

: Type>Form-Output+CR	( a n -- )
	KeyButtonQueue swap 2dup + >r cmove 13 r> c!
;

: AddTypedText		( -- )		\ adds text directly typed into TxtBox
	Output 0 <>
		if GetTypedInput dup 2 >
			if   2dup XYappend SetOutputText
			     Type>Form-Output WinPause 100 ms WinPause
			else 2drop
			then
		then
;

: AddTypedText+$	( a n -- )	\ adds text directly typed into TxtBox and String
	Output 0 <>
		if GetTypedInput dup 2 >
			if   2dup XYappend SetOutputText
			     Type2>Form-Output WinPause 100 ms WinPause
			else 2drop Type>Form-Output
			then
		else 2drop
		then
;

: AddTypedText+CR	( -- )		\ adds text directly typed into TxtBox, adds CR anyway
	Output 0 <>
		if GetTypedInput dup 2 >
			if   2dup XYappend SetOutputText
			     Type>Form-Output+CR WinPause 100 ms WinPause
			else 2drop 13  to Form-Output
			then
		then
;

: cr>TextBox	( -- )
		crlf$ count (type>TextBox)
		RowX l/p 3 - > if MoveLineUp then	\ added for scrolling DB 2009-0403
 		GetNewRowStart to RowEnd
;

: type>TextBox	( adr len -- )
	(type>TextBox)
;


: emit>TextBoxAndScreen ( char -- )
		dup emit>TextBox oEmit @ execute 
;

: type>TextBoxAndScreen ( adr len -- )
		2dup type>TextBox oType @ execute 
;

: cr>TextBoxAndScreen   ( -- )
		cr>TextBox oCr @ execute 
;

: GOTOXY>TextBox	( x y -- )
	dup RowX > if  RowEnd 1+ to RowStart then
	l/p 2 - min to RowX to ColoumnY 
;

: GOTOXY>TextBoxAndScreen
	2dup GOTOXY>TextBox oGOTOXY @ execute	\ without oGOTOXY no echon on console !
;

: CLS>TextBox ( -- )
	InitOutputBuffer TheOutputBuffer SetText: Output
;

: CLS>TextBoxAndScreen
	CLS>TextBox oCLS @ execute		\ without oCLS only one line, no results
;


\ external


: >TextBox		( -- ) \ switches screen output to a TextBox only
		Output              \ (Output must be set)
		if   SaveRedirection
		     ['] emit>TextBox ['] type>TextBox ['] cr>TextBox ['] GOTOXY>TextBox
		     ['] CLS>TextBox
		     SetRedirection InitOutputBuffer
		then ;

: >TextBoxAndScreen ( -- ) \ switches screen output to a TextBox
		        \ and keeps the output on the screen
		Output              
		if   SaveRedirection
		     ['] emit>TextBoxAndScreen   ['] type>TextBoxAndScreen ['] cr>TextBoxAndScreen
		     ['] GOTOXY>TextBoxAndScreen ['] CLS>TextBoxAndScreen
		     SetRedirection InitOutputBuffer
		then ;

: IsOutput	( MtxtBox -- )
	to Output 
	c/l c/lmax min l/p Rowmax min * 1- to OutputSize
	InitOutputBuffer
	GetColRow oL/P ! oC/L !
	c/l l/p SetColRow
	>TextBoxAndScreen
;

: >screen	( -- ) \ switches back to normal output
		RestoreRedirection ;


\ for W32F 6.14	from ExUtils.f old etc. DB 10-0605

-1 value WM_WIN32FORTH                          \ w32f
\ *G This windows message definition for WM_WIN32FORTH, provides a way for
\ ** multiple Win32Forth applications to interact between themselves
\ ** while running.

defer win32forth-message

\ changed September 6th, 2003 - 17:30 dbu
7 proc SendMessageTimeout
: _win32forth-message ( lParam wParam -- )      \ w32f intern
        2>R 0 SP@ 2 ( ms ) SMTO_ABORTIFHUNG 2R>
        WM_WIN32FORTH HWND_BROADCAST Call SendMessageTimeout 2drop ;

' _win32forth-message is win32forth-message ( lParam wParam -- ) \ w32f
\ *G WIN32FORTH-MESSAGE allows a unique message to be broadcast to all
\ ** currently running instances of Win32Forth.
\ *P 'wParam' is the application specific sub-message that each instance can
\ *P ** use to determine if it should handle the message.
\ ** 'lParam' is available to pass specific information between instances
\ ** of Win32Forth.
\ *P Note: The message is not only broadcast to to all currently running
\ ** instances of Win32Forth. It is brodcast to ALL windows in the system.
\ ** So it may be rather slow...

comment:
The following routines uses shared memory for interprocess communication. Uses
the last 1k for info passing, arbitrarily reserving the first 24 bytes for
process ids. First cell will identify visualFORTH. Second cell identifies Project Manager.
comment;

newproc FF_ACTIVATE  \ sent on startup
newproc WANT_FORMS   \ request forms
newproc FORMS_SENT   \ acknowledge


0 value ed-ptr
1024 constant msg-buff-len
msg-buff-len  6 cells - constant param-max

4096 ( + 4096 / 4096 * ) constant ed-size   \ multiples of 4k bytes - from editor_io.f old


ed-size msg-buff-len - constant msg-offset  \ start of message buffer for visualFORTH
102378530 constant FFormID              \ identify visualFORTH
872015346 constant ProMgrID             \ identify Project Manager
0 value msg-buffer                      \ pointer to buffer, set at runtime
0 value param-buffer

: init-msg-buffer  ( -- )
                  ed-ptr ?dup
                  if    msg-offset + to msg-buffer   \ run time initialization
                        msg-buffer [ 6 cells ] LITERAL + to param-buffer
                  then  ;

Initialization-chain Chain-add init-msg-buffer
init-msg-buffer

: ?vF-started   ( -- f )
                   msg-buffer FFormID = ;

: vF-started    ( f -- )
                   if         FFormID
                   else       0
                   then       to msg-buffer ;
comment:


: ?promgr-started ( -- f )
                   msg-buffer cell+ @ ProMgrID = ;

: promgr-started   ( f -- )
                   if         ProMgrID
                   else       0
                   then       msg-buffer cell+ ! ;
comment;

\ end of for W32F form ExUtils.f old




: rem-sendkey   ( c1 -- )          \ pass a key to a remote Forth
                WM_KEY win32forth-message 
\ cr s"  rem-sendkey not yet available " type cr
;

variable +CR
+CR off

: "CR K_CR rem-sendkey  +CR off ;

: "+CR +CR @ if "CR then ;

: "-CR +CR @ 0= if "CR then ;

: "Tab K_TAB rem-sendkey ;  

: "Forth        ( a1 n1 -- )
                bounds    ?DO  I c@ rem-sendkey
                          0 ms \ release control to OS for a moment
                          LOOP \ 0x0D rem-sendkey
		"+CR
;

: "Forth+CR "Forth "CR ;
: "Forth+NL "Forth+CR "Tab ;

Forth module
